# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations

from typing import MutableSequence

import proto  # type: ignore

from google.ads.googleads.v20.common.types import bidding
from google.ads.googleads.v20.common.types import custom_parameter
from google.ads.googleads.v20.common.types import frequency_cap
from google.ads.googleads.v20.common.types import (
    real_time_bidding_setting as gagc_real_time_bidding_setting,
)
from google.ads.googleads.v20.common.types import (
    targeting_setting as gagc_targeting_setting,
)
from google.ads.googleads.v20.enums.types import (
    ad_serving_optimization_status as gage_ad_serving_optimization_status,
)
from google.ads.googleads.v20.enums.types import (
    advertising_channel_sub_type as gage_advertising_channel_sub_type,
)
from google.ads.googleads.v20.enums.types import (
    advertising_channel_type as gage_advertising_channel_type,
)
from google.ads.googleads.v20.enums.types import app_campaign_app_store
from google.ads.googleads.v20.enums.types import (
    app_campaign_bidding_strategy_goal_type,
)
from google.ads.googleads.v20.enums.types import (
    asset_automation_status as gage_asset_automation_status,
)
from google.ads.googleads.v20.enums.types import (
    asset_automation_type as gage_asset_automation_type,
)
from google.ads.googleads.v20.enums.types import asset_field_type
from google.ads.googleads.v20.enums.types import asset_set_type
from google.ads.googleads.v20.enums.types import (
    bidding_strategy_system_status as gage_bidding_strategy_system_status,
)
from google.ads.googleads.v20.enums.types import (
    bidding_strategy_type as gage_bidding_strategy_type,
)
from google.ads.googleads.v20.enums.types import brand_safety_suitability
from google.ads.googleads.v20.enums.types import campaign_experiment_type
from google.ads.googleads.v20.enums.types import campaign_keyword_match_type
from google.ads.googleads.v20.enums.types import campaign_primary_status
from google.ads.googleads.v20.enums.types import campaign_primary_status_reason
from google.ads.googleads.v20.enums.types import campaign_serving_status
from google.ads.googleads.v20.enums.types import campaign_status
from google.ads.googleads.v20.enums.types import eu_political_advertising_status
from google.ads.googleads.v20.enums.types import (
    listing_type as gage_listing_type,
)
from google.ads.googleads.v20.enums.types import (
    location_source_type as gage_location_source_type,
)
from google.ads.googleads.v20.enums.types import (
    negative_geo_target_type as gage_negative_geo_target_type,
)
from google.ads.googleads.v20.enums.types import non_skippable_max_duration
from google.ads.googleads.v20.enums.types import non_skippable_min_duration
from google.ads.googleads.v20.enums.types import optimization_goal_type
from google.ads.googleads.v20.enums.types import (
    payment_mode as gage_payment_mode,
)
from google.ads.googleads.v20.enums.types import performance_max_upgrade_status
from google.ads.googleads.v20.enums.types import (
    positive_geo_target_type as gage_positive_geo_target_type,
)
from google.ads.googleads.v20.enums.types import (
    vanity_pharma_display_url_mode as gage_vanity_pharma_display_url_mode,
)
from google.ads.googleads.v20.enums.types import (
    vanity_pharma_text as gage_vanity_pharma_text,
)
from google.ads.googleads.v20.enums.types import video_ad_format_restriction


__protobuf__ = proto.module(
    package="google.ads.googleads.v20.resources",
    marshal="google.ads.googleads.v20",
    manifest={
        "Campaign",
    },
)


class Campaign(proto.Message):
    r"""A campaign.

    This message has `oneof`_ fields (mutually exclusive fields).
    For each oneof, at most one member field can be set at the same time.
    Setting any member of the oneof automatically clears all other
    members.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        resource_name (str):
            Immutable. The resource name of the campaign. Campaign
            resource names have the form:

            ``customers/{customer_id}/campaigns/{campaign_id}``
        id (int):
            Output only. The ID of the campaign.

            This field is a member of `oneof`_ ``_id``.
        name (str):
            The name of the campaign.

            This field is required and should not be empty
            when creating new campaigns.

            It must not contain any null (code point 0x0),
            NL line feed (code point 0xA) or carriage return
            (code point 0xD) characters.

            This field is a member of `oneof`_ ``_name``.
        primary_status (google.ads.googleads.v20.enums.types.CampaignPrimaryStatusEnum.CampaignPrimaryStatus):
            Output only. The primary status of the
            campaign.
            Provides insight into why a campaign is not
            serving or not serving optimally. Modification
            to the campaign and its related entities might
            take a while to be reflected in this status.
        primary_status_reasons (MutableSequence[google.ads.googleads.v20.enums.types.CampaignPrimaryStatusReasonEnum.CampaignPrimaryStatusReason]):
            Output only. The primary status reasons of
            the campaign.
            Provides insight into why a campaign is not
            serving or not serving optimally. These reasons
            are aggregated to determine an overall
            CampaignPrimaryStatus.
        status (google.ads.googleads.v20.enums.types.CampaignStatusEnum.CampaignStatus):
            The status of the campaign.

            When a new campaign is added, the status
            defaults to ENABLED.
        serving_status (google.ads.googleads.v20.enums.types.CampaignServingStatusEnum.CampaignServingStatus):
            Output only. The ad serving status of the
            campaign.
        bidding_strategy_system_status (google.ads.googleads.v20.enums.types.BiddingStrategySystemStatusEnum.BiddingStrategySystemStatus):
            Output only. The system status of the
            campaign's bidding strategy.
        ad_serving_optimization_status (google.ads.googleads.v20.enums.types.AdServingOptimizationStatusEnum.AdServingOptimizationStatus):
            The ad serving optimization status of the
            campaign.
        advertising_channel_type (google.ads.googleads.v20.enums.types.AdvertisingChannelTypeEnum.AdvertisingChannelType):
            Immutable. The primary serving target for ads within the
            campaign. The targeting options can be refined in
            ``network_settings``.

            This field is required and should not be empty when creating
            new campaigns.

            Can be set only when creating campaigns. After the campaign
            is created, the field can not be changed.
        advertising_channel_sub_type (google.ads.googleads.v20.enums.types.AdvertisingChannelSubTypeEnum.AdvertisingChannelSubType):
            Immutable. Optional refinement to
            ``advertising_channel_type``. Must be a valid sub-type of
            the parent channel type.

            Can be set only when creating campaigns. After campaign is
            created, the field can not be changed.
        tracking_url_template (str):
            The URL template for constructing a tracking
            URL.

            This field is a member of `oneof`_ ``_tracking_url_template``.
        url_custom_parameters (MutableSequence[google.ads.googleads.v20.common.types.CustomParameter]):
            The list of mappings used to substitute custom parameter
            tags in a ``tracking_url_template``, ``final_urls``, or
            ``mobile_final_urls``.
        local_services_campaign_settings (google.ads.googleads.v20.resources.types.Campaign.LocalServicesCampaignSettings):
            The Local Services Campaign related settings.
        travel_campaign_settings (google.ads.googleads.v20.resources.types.Campaign.TravelCampaignSettings):
            Settings for Travel campaign.
        demand_gen_campaign_settings (google.ads.googleads.v20.resources.types.Campaign.DemandGenCampaignSettings):
            Settings for Demand Gen campaign.
        video_campaign_settings (google.ads.googleads.v20.resources.types.Campaign.VideoCampaignSettings):
            Settings for Video campaign.
        pmax_campaign_settings (google.ads.googleads.v20.resources.types.Campaign.PmaxCampaignSettings):
            Settings for Performance Max campaign.
        real_time_bidding_setting (google.ads.googleads.v20.common.types.RealTimeBiddingSetting):
            Settings for Real-Time Bidding, a feature
            only available for campaigns targeting the Ad
            Exchange network.
        network_settings (google.ads.googleads.v20.resources.types.Campaign.NetworkSettings):
            The network settings for the campaign.
        hotel_setting (google.ads.googleads.v20.resources.types.Campaign.HotelSettingInfo):
            Immutable. The hotel setting for the
            campaign.
        dynamic_search_ads_setting (google.ads.googleads.v20.resources.types.Campaign.DynamicSearchAdsSetting):
            The setting for controlling Dynamic Search
            Ads (DSA).
        shopping_setting (google.ads.googleads.v20.resources.types.Campaign.ShoppingSetting):
            The setting for controlling Shopping
            campaigns.
        targeting_setting (google.ads.googleads.v20.common.types.TargetingSetting):
            Setting for targeting related features.
        audience_setting (google.ads.googleads.v20.resources.types.Campaign.AudienceSetting):
            Immutable. Setting for audience related
            features.

            This field is a member of `oneof`_ ``_audience_setting``.
        geo_target_type_setting (google.ads.googleads.v20.resources.types.Campaign.GeoTargetTypeSetting):
            The setting for ads geotargeting.
        local_campaign_setting (google.ads.googleads.v20.resources.types.Campaign.LocalCampaignSetting):
            The setting for local campaign.
        app_campaign_setting (google.ads.googleads.v20.resources.types.Campaign.AppCampaignSetting):
            The setting related to App Campaign.
        labels (MutableSequence[str]):
            Output only. The resource names of labels
            attached to this campaign.
        experiment_type (google.ads.googleads.v20.enums.types.CampaignExperimentTypeEnum.CampaignExperimentType):
            Output only. The type of campaign: normal,
            draft, or experiment.
        base_campaign (str):
            Output only. The resource name of the base campaign of a
            draft or experiment campaign. For base campaigns, this is
            equal to ``resource_name``.

            This field is read-only.

            This field is a member of `oneof`_ ``_base_campaign``.
        campaign_budget (str):
            The resource name of the campaign budget of
            the campaign.

            This field is a member of `oneof`_ ``_campaign_budget``.
        bidding_strategy_type (google.ads.googleads.v20.enums.types.BiddingStrategyTypeEnum.BiddingStrategyType):
            Output only. The type of bidding strategy.

            A bidding strategy can be created by setting either the
            bidding scheme to create a standard bidding strategy or the
            ``bidding_strategy`` field to create a portfolio bidding
            strategy.

            This field is read-only.
        accessible_bidding_strategy (str):
            Output only. Resource name of AccessibleBiddingStrategy, a
            read-only view of the unrestricted attributes of the
            attached portfolio bidding strategy identified by
            'bidding_strategy'. Empty, if the campaign does not use a
            portfolio strategy. Unrestricted strategy attributes are
            available to all customers with whom the strategy is shared
            and are read from the AccessibleBiddingStrategy resource. In
            contrast, restricted attributes are only available to the
            owner customer of the strategy and their managers.
            Restricted attributes can only be read from the
            BiddingStrategy resource.
        start_date (str):
            The date when campaign started in serving
            customer's timezone in YYYY-MM-DD format.

            This field is a member of `oneof`_ ``_start_date``.
        campaign_group (str):
            The resource name of the campaign group that
            this campaign belongs to.

            This field is a member of `oneof`_ ``_campaign_group``.
        end_date (str):
            The last day of the campaign in serving
            customer's timezone in YYYY-MM-DD format. On
            create, defaults to 2037-12-30, which means the
            campaign will run indefinitely. To set an
            existing campaign to run indefinitely, set this
            field to 2037-12-30.

            This field is a member of `oneof`_ ``_end_date``.
        final_url_suffix (str):
            Suffix used to append query parameters to
            landing pages that are served with parallel
            tracking.

            This field is a member of `oneof`_ ``_final_url_suffix``.
        frequency_caps (MutableSequence[google.ads.googleads.v20.common.types.FrequencyCapEntry]):
            A list that limits how often each user will
            see this campaign's ads.
        video_brand_safety_suitability (google.ads.googleads.v20.enums.types.BrandSafetySuitabilityEnum.BrandSafetySuitability):
            Brand Safety setting at the individual
            campaign level. Allows for selecting an
            inventory type to show your ads on content that
            is the right fit for your brand. See
            https://support.google.com/google-ads/answer/7515513.
        vanity_pharma (google.ads.googleads.v20.resources.types.Campaign.VanityPharma):
            Describes how unbranded pharma ads will be
            displayed.
        selective_optimization (google.ads.googleads.v20.resources.types.Campaign.SelectiveOptimization):
            Selective optimization setting for this campaign, which
            includes a set of conversion actions to optimize this
            campaign towards. This feature only applies to app campaigns
            that use MULTI_CHANNEL as AdvertisingChannelType and
            APP_CAMPAIGN or APP_CAMPAIGN_FOR_ENGAGEMENT as
            AdvertisingChannelSubType.
        optimization_goal_setting (google.ads.googleads.v20.resources.types.Campaign.OptimizationGoalSetting):
            Optimization goal setting for this campaign,
            which includes a set of optimization goal types.
        tracking_setting (google.ads.googleads.v20.resources.types.Campaign.TrackingSetting):
            Output only. Campaign-level settings for
            tracking information.
        payment_mode (google.ads.googleads.v20.enums.types.PaymentModeEnum.PaymentMode):
            Payment mode for the campaign.
        optimization_score (float):
            Output only. Optimization score of the
            campaign.
            Optimization score is an estimate of how well a
            campaign is set to perform. It ranges from 0%
            (0.0) to 100% (1.0), with 100% indicating that
            the campaign is performing at full potential.
            This field is null for unscored campaigns.

            See "About optimization score" at
            https://support.google.com/google-ads/answer/9061546.

            This field is read-only.

            This field is a member of `oneof`_ ``_optimization_score``.
        excluded_parent_asset_field_types (MutableSequence[google.ads.googleads.v20.enums.types.AssetFieldTypeEnum.AssetFieldType]):
            The asset field types that should be excluded
            from this campaign. Asset links with these field
            types will not be inherited by this campaign
            from the upper level.
        excluded_parent_asset_set_types (MutableSequence[google.ads.googleads.v20.enums.types.AssetSetTypeEnum.AssetSetType]):
            The asset set types that should be excluded from this
            campaign. Asset set links with these types will not be
            inherited by this campaign from the upper level. Location
            group types (GMB_DYNAMIC_LOCATION_GROUP,
            CHAIN_DYNAMIC_LOCATION_GROUP, and STATIC_LOCATION_GROUP) are
            child types of LOCATION_SYNC. Therefore, if LOCATION_SYNC is
            set for this field, all location group asset sets are not
            allowed to be linked to this campaign, and all Location
            Extension (LE) and Affiliate Location Extensions (ALE) will
            not be served under this campaign. Only LOCATION_SYNC is
            currently supported.
        url_expansion_opt_out (bool):
            Represents opting out of URL expansion to
            more targeted URLs. If opted out (true), only
            the final URLs in the asset group or URLs
            specified in the advertiser's Google Merchant
            Center or business data feeds are targeted. If
            opted in (false), the entire domain will be
            targeted. This field can only be set for
            Performance Max campaigns, where the default
            value is false.

            This field is a member of `oneof`_ ``_url_expansion_opt_out``.
        performance_max_upgrade (google.ads.googleads.v20.resources.types.Campaign.PerformanceMaxUpgrade):
            Output only. Information about campaigns
            being upgraded to Performance Max.
        hotel_property_asset_set (str):
            Immutable. The resource name for a set of
            hotel properties for Performance Max for travel
            goals campaigns.

            This field is a member of `oneof`_ ``_hotel_property_asset_set``.
        listing_type (google.ads.googleads.v20.enums.types.ListingTypeEnum.ListingType):
            Immutable. Listing type of ads served for
            this campaign. Field is restricted for usage
            with Performance Max campaigns.

            This field is a member of `oneof`_ ``_listing_type``.
        asset_automation_settings (MutableSequence[google.ads.googleads.v20.resources.types.Campaign.AssetAutomationSetting]):
            Contains the opt-in/out status of each
            AssetAutomationType. See documentation of each
            asset automation type enum for default opt
            in/out behavior.
        keyword_match_type (google.ads.googleads.v20.enums.types.CampaignKeywordMatchTypeEnum.CampaignKeywordMatchType):
            Keyword match type of Campaign. Set to BROAD
            to set broad matching for all keywords in a
            campaign.
        brand_guidelines_enabled (bool):
            Immutable. Whether Brand Guidelines are
            enabled for this Campaign. Only applicable to
            Performance Max campaigns. If enabled, business
            name and logo assets must be linked as
            CampaignAssets instead of AssetGroupAssets.

            Writable only at campaign creation. Set to true
            to enable Brand Guidelines when creating a new
            Performance Max campaign.

            Immutable after creation. This field cannot be
            modified using standard update operations after
            the campaign has been created.

            For existing campaigns: To enable Brand
            Guidelines on a campaign after it has been
            created, use the
            CampaignService.EnablePMaxBrandGuidelines
            method, which is a separate operation. It is not
            possible to disable Brand Guidelines for an
            existing campaign.

            Incompatible with Travel Goals: This feature is
            not supported for Performance Max campaigns with
            Travel Goals. Attempting to set this field to
            true for a Travel Goals campaign will result in
            an error.

            This field is a member of `oneof`_ ``_brand_guidelines_enabled``.
        brand_guidelines (google.ads.googleads.v20.resources.types.Campaign.BrandGuidelines):
            These settings control how your brand appears
            in automatically generated assets and formats
            within this campaign. Note: These settings can
            only be used for Performance Max campaigns that
            have Brand Guidelines enabled.
        contains_eu_political_advertising (google.ads.googleads.v20.enums.types.EuPoliticalAdvertisingStatusEnum.EuPoliticalAdvertisingStatus):
            The advertiser should self-declare whether
            this campaign contains political advertising
            content targeted towards the European Union.
        bidding_strategy (str):
            The resource name of the portfolio bidding
            strategy used by the campaign.

            This field is a member of `oneof`_ ``campaign_bidding_strategy``.
        commission (google.ads.googleads.v20.common.types.Commission):
            Commission is an automatic bidding strategy
            in which the advertiser pays a certain portion
            of the conversion value.

            This field is a member of `oneof`_ ``campaign_bidding_strategy``.
        manual_cpa (google.ads.googleads.v20.common.types.ManualCpa):
            Standard Manual CPA bidding strategy.
            Manual bidding strategy that allows advertiser
            to set the bid per advertiser-specified action.
            Supported only for Local Services campaigns.

            This field is a member of `oneof`_ ``campaign_bidding_strategy``.
        manual_cpc (google.ads.googleads.v20.common.types.ManualCpc):
            Standard Manual CPC bidding strategy.
            Manual click-based bidding where user pays per
            click.

            This field is a member of `oneof`_ ``campaign_bidding_strategy``.
        manual_cpm (google.ads.googleads.v20.common.types.ManualCpm):
            Standard Manual CPM bidding strategy.
            Manual impression-based bidding where user pays
            per thousand impressions.

            This field is a member of `oneof`_ ``campaign_bidding_strategy``.
        manual_cpv (google.ads.googleads.v20.common.types.ManualCpv):
            A bidding strategy that pays a configurable
            amount per video view.

            This field is a member of `oneof`_ ``campaign_bidding_strategy``.
        maximize_conversions (google.ads.googleads.v20.common.types.MaximizeConversions):
            Standard Maximize Conversions bidding
            strategy that automatically maximizes number of
            conversions while spending your budget.

            This field is a member of `oneof`_ ``campaign_bidding_strategy``.
        maximize_conversion_value (google.ads.googleads.v20.common.types.MaximizeConversionValue):
            Standard Maximize Conversion Value bidding
            strategy that automatically sets bids to
            maximize revenue while spending your budget.

            This field is a member of `oneof`_ ``campaign_bidding_strategy``.
        target_cpa (google.ads.googleads.v20.common.types.TargetCpa):
            Standard Target CPA bidding strategy that
            automatically sets bids to help get as many
            conversions as possible at the target
            cost-per-acquisition (CPA) you set.

            This field is a member of `oneof`_ ``campaign_bidding_strategy``.
        target_impression_share (google.ads.googleads.v20.common.types.TargetImpressionShare):
            Target Impression Share bidding strategy. An
            automated bidding strategy that sets bids to
            achieve a chosen percentage of impressions.

            This field is a member of `oneof`_ ``campaign_bidding_strategy``.
        target_roas (google.ads.googleads.v20.common.types.TargetRoas):
            Standard Target ROAS bidding strategy that
            automatically maximizes revenue while averaging
            a specific target return on ad spend (ROAS).

            This field is a member of `oneof`_ ``campaign_bidding_strategy``.
        target_spend (google.ads.googleads.v20.common.types.TargetSpend):
            Standard Target Spend bidding strategy that
            automatically sets your bids to help get as many
            clicks as possible within your budget.

            This field is a member of `oneof`_ ``campaign_bidding_strategy``.
        percent_cpc (google.ads.googleads.v20.common.types.PercentCpc):
            Standard Percent Cpc bidding strategy where
            bids are a fraction of the advertised price for
            some good or service.

            This field is a member of `oneof`_ ``campaign_bidding_strategy``.
        target_cpm (google.ads.googleads.v20.common.types.TargetCpm):
            A bidding strategy that automatically
            optimizes cost per thousand impressions.

            This field is a member of `oneof`_ ``campaign_bidding_strategy``.
        fixed_cpm (google.ads.googleads.v20.common.types.FixedCpm):
            A manual bidding strategy with a fixed CPM.

            This field is a member of `oneof`_ ``campaign_bidding_strategy``.
        target_cpv (google.ads.googleads.v20.common.types.TargetCpv):
            An automated bidding strategy that sets bids
            to optimize performance given the target CPV you
            set.

            This field is a member of `oneof`_ ``campaign_bidding_strategy``.
    """

    class PerformanceMaxUpgrade(proto.Message):
        r"""Information about a campaign being upgraded to Performance
        Max.

        Attributes:
            performance_max_campaign (str):
                Output only. The resource name of the
                Performance Max campaign the campaign is
                upgraded to.
            pre_upgrade_campaign (str):
                Output only. The resource name of the legacy
                campaign upgraded to Performance Max.
            status (google.ads.googleads.v20.enums.types.PerformanceMaxUpgradeStatusEnum.PerformanceMaxUpgradeStatus):
                Output only. The upgrade status of a campaign
                requested to be upgraded to Performance Max.
        """

        performance_max_campaign: str = proto.Field(
            proto.STRING,
            number=1,
        )
        pre_upgrade_campaign: str = proto.Field(
            proto.STRING,
            number=2,
        )
        status: (
            performance_max_upgrade_status.PerformanceMaxUpgradeStatusEnum.PerformanceMaxUpgradeStatus
        ) = proto.Field(
            proto.ENUM,
            number=3,
            enum=performance_max_upgrade_status.PerformanceMaxUpgradeStatusEnum.PerformanceMaxUpgradeStatus,
        )

    class NetworkSettings(proto.Message):
        r"""The network settings for the campaign.

        .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

        Attributes:
            target_google_search (bool):
                Whether ads will be served with google.com
                search results.

                This field is a member of `oneof`_ ``_target_google_search``.
            target_search_network (bool):
                Whether ads will be served on partner sites in the Google
                Search Network (requires ``target_google_search`` to also be
                ``true``).

                This field is a member of `oneof`_ ``_target_search_network``.
            target_content_network (bool):
                Whether ads will be served on specified
                placements in the Google Display Network.
                Placements are specified using the Placement
                criterion.

                This field is a member of `oneof`_ ``_target_content_network``.
            target_partner_search_network (bool):
                Whether ads will be served on the Google
                Partner Network. This is available only to some
                select Google partner accounts.

                This field is a member of `oneof`_ ``_target_partner_search_network``.
            target_youtube (bool):
                Whether ads will be served on YouTube.

                This field is a member of `oneof`_ ``_target_youtube``.
            target_google_tv_network (bool):
                Whether ads will be served on the Google TV
                network.

                This field is a member of `oneof`_ ``_target_google_tv_network``.
        """

        target_google_search: bool = proto.Field(
            proto.BOOL,
            number=5,
            optional=True,
        )
        target_search_network: bool = proto.Field(
            proto.BOOL,
            number=6,
            optional=True,
        )
        target_content_network: bool = proto.Field(
            proto.BOOL,
            number=7,
            optional=True,
        )
        target_partner_search_network: bool = proto.Field(
            proto.BOOL,
            number=8,
            optional=True,
        )
        target_youtube: bool = proto.Field(
            proto.BOOL,
            number=9,
            optional=True,
        )
        target_google_tv_network: bool = proto.Field(
            proto.BOOL,
            number=10,
            optional=True,
        )

    class HotelSettingInfo(proto.Message):
        r"""Campaign-level settings for hotel ads.

        .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

        Attributes:
            hotel_center_id (int):
                Immutable. The linked Hotel Center account.

                This field is a member of `oneof`_ ``_hotel_center_id``.
        """

        hotel_center_id: int = proto.Field(
            proto.INT64,
            number=2,
            optional=True,
        )

    class DynamicSearchAdsSetting(proto.Message):
        r"""The setting for controlling Dynamic Search Ads (DSA).

        .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

        Attributes:
            domain_name (str):
                Required. The Internet domain name that this
                setting represents, for example, "google.com" or
                "www.google.com".
            language_code (str):
                Required. The language code specifying the
                language of the domain, for example, "en".
            use_supplied_urls_only (bool):
                Whether the campaign uses advertiser supplied
                URLs exclusively.

                This field is a member of `oneof`_ ``_use_supplied_urls_only``.
        """

        domain_name: str = proto.Field(
            proto.STRING,
            number=6,
        )
        language_code: str = proto.Field(
            proto.STRING,
            number=7,
        )
        use_supplied_urls_only: bool = proto.Field(
            proto.BOOL,
            number=8,
            optional=True,
        )

    class ShoppingSetting(proto.Message):
        r"""The setting for Shopping campaigns. Defines the universe of
        products that can be advertised by the campaign, and how this
        campaign interacts with other Shopping campaigns.


        .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

        Attributes:
            merchant_id (int):
                ID of the Merchant Center account.
                This field is required for create operations.
                This field is immutable for Shopping campaigns.

                This field is a member of `oneof`_ ``_merchant_id``.
            feed_label (str):
                Feed label of products to include in the campaign. Only one
                of feed_label or sales_country can be set. If used instead
                of sales_country, the feed_label field accepts country codes
                in the same format for example: 'XX'. Otherwise can be any
                string used for feed label in Google Merchant Center.
            campaign_priority (int):
                Priority of the campaign. Campaigns with
                numerically higher priorities take precedence
                over those with lower priorities. This field is
                required for Shopping campaigns, with values
                between 0 and 2, inclusive.
                This field is optional for Smart Shopping
                campaigns, but must be equal to 3 if set.

                This field is a member of `oneof`_ ``_campaign_priority``.
            enable_local (bool):
                Whether to include local products.

                This field is a member of `oneof`_ ``_enable_local``.
            use_vehicle_inventory (bool):
                Immutable. Whether to target Vehicle Listing inventory. This
                field is supported only in Smart Shopping Campaigns. For
                setting Vehicle Listing inventory in Performance Max
                campaigns, use ``listing_type`` instead.
            advertising_partner_ids (MutableSequence[int]):
                Immutable. The list of Google Ads accounts
                IDs of advertising partners cooperating within
                the campaign. This feature is currently
                available only for accounts having an
                advertising partner link. This feature is
                currently supported only for Performance Max,
                Shopping, Search and Demand Gen campaign types.
            disable_product_feed (bool):
                Disable the optional product feed. This field
                is currently supported only for Demand Gen
                campaigns. See
                https://support.google.com/google-ads/answer/13721750
                to learn more about this feature.

                This field is a member of `oneof`_ ``_disable_product_feed``.
        """

        merchant_id: int = proto.Field(
            proto.INT64,
            number=5,
            optional=True,
        )
        feed_label: str = proto.Field(
            proto.STRING,
            number=10,
        )
        campaign_priority: int = proto.Field(
            proto.INT32,
            number=7,
            optional=True,
        )
        enable_local: bool = proto.Field(
            proto.BOOL,
            number=8,
            optional=True,
        )
        use_vehicle_inventory: bool = proto.Field(
            proto.BOOL,
            number=9,
        )
        advertising_partner_ids: MutableSequence[int] = proto.RepeatedField(
            proto.INT64,
            number=11,
        )
        disable_product_feed: bool = proto.Field(
            proto.BOOL,
            number=12,
            optional=True,
        )

    class TrackingSetting(proto.Message):
        r"""Campaign-level settings for tracking information.

        .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

        Attributes:
            tracking_url (str):
                Output only. The url used for dynamic
                tracking.

                This field is a member of `oneof`_ ``_tracking_url``.
        """

        tracking_url: str = proto.Field(
            proto.STRING,
            number=2,
            optional=True,
        )

    class GeoTargetTypeSetting(proto.Message):
        r"""Represents a collection of settings related to ads
        geotargeting.

        Attributes:
            positive_geo_target_type (google.ads.googleads.v20.enums.types.PositiveGeoTargetTypeEnum.PositiveGeoTargetType):
                The setting used for positive geotargeting in
                this particular campaign.
            negative_geo_target_type (google.ads.googleads.v20.enums.types.NegativeGeoTargetTypeEnum.NegativeGeoTargetType):
                The setting used for negative geotargeting in
                this particular campaign.
        """

        positive_geo_target_type: (
            gage_positive_geo_target_type.PositiveGeoTargetTypeEnum.PositiveGeoTargetType
        ) = proto.Field(
            proto.ENUM,
            number=1,
            enum=gage_positive_geo_target_type.PositiveGeoTargetTypeEnum.PositiveGeoTargetType,
        )
        negative_geo_target_type: (
            gage_negative_geo_target_type.NegativeGeoTargetTypeEnum.NegativeGeoTargetType
        ) = proto.Field(
            proto.ENUM,
            number=2,
            enum=gage_negative_geo_target_type.NegativeGeoTargetTypeEnum.NegativeGeoTargetType,
        )

    class LocalCampaignSetting(proto.Message):
        r"""Campaign setting for local campaigns.

        Attributes:
            location_source_type (google.ads.googleads.v20.enums.types.LocationSourceTypeEnum.LocationSourceType):
                The location source type for this local
                campaign.
        """

        location_source_type: (
            gage_location_source_type.LocationSourceTypeEnum.LocationSourceType
        ) = proto.Field(
            proto.ENUM,
            number=1,
            enum=gage_location_source_type.LocationSourceTypeEnum.LocationSourceType,
        )

    class AppCampaignSetting(proto.Message):
        r"""Campaign-level settings for App Campaigns.

        .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

        Attributes:
            bidding_strategy_goal_type (google.ads.googleads.v20.enums.types.AppCampaignBiddingStrategyGoalTypeEnum.AppCampaignBiddingStrategyGoalType):
                Represents the goal which the bidding
                strategy of this app campaign should optimize
                towards.
            app_id (str):
                Immutable. A string that uniquely identifies
                a mobile application.

                This field is a member of `oneof`_ ``_app_id``.
            app_store (google.ads.googleads.v20.enums.types.AppCampaignAppStoreEnum.AppCampaignAppStore):
                Immutable. The application store that
                distributes this specific app.
        """

        bidding_strategy_goal_type: (
            app_campaign_bidding_strategy_goal_type.AppCampaignBiddingStrategyGoalTypeEnum.AppCampaignBiddingStrategyGoalType
        ) = proto.Field(
            proto.ENUM,
            number=1,
            enum=app_campaign_bidding_strategy_goal_type.AppCampaignBiddingStrategyGoalTypeEnum.AppCampaignBiddingStrategyGoalType,
        )
        app_id: str = proto.Field(
            proto.STRING,
            number=4,
            optional=True,
        )
        app_store: (
            app_campaign_app_store.AppCampaignAppStoreEnum.AppCampaignAppStore
        ) = proto.Field(
            proto.ENUM,
            number=3,
            enum=app_campaign_app_store.AppCampaignAppStoreEnum.AppCampaignAppStore,
        )

    class VanityPharma(proto.Message):
        r"""Describes how unbranded pharma ads will be displayed.

        Attributes:
            vanity_pharma_display_url_mode (google.ads.googleads.v20.enums.types.VanityPharmaDisplayUrlModeEnum.VanityPharmaDisplayUrlMode):
                The display mode for vanity pharma URLs.
            vanity_pharma_text (google.ads.googleads.v20.enums.types.VanityPharmaTextEnum.VanityPharmaText):
                The text that will be displayed in display
                URL of the text ad when website description is
                the selected display mode for vanity pharma
                URLs.
        """

        vanity_pharma_display_url_mode: (
            gage_vanity_pharma_display_url_mode.VanityPharmaDisplayUrlModeEnum.VanityPharmaDisplayUrlMode
        ) = proto.Field(
            proto.ENUM,
            number=1,
            enum=gage_vanity_pharma_display_url_mode.VanityPharmaDisplayUrlModeEnum.VanityPharmaDisplayUrlMode,
        )
        vanity_pharma_text: (
            gage_vanity_pharma_text.VanityPharmaTextEnum.VanityPharmaText
        ) = proto.Field(
            proto.ENUM,
            number=2,
            enum=gage_vanity_pharma_text.VanityPharmaTextEnum.VanityPharmaText,
        )

    class SelectiveOptimization(proto.Message):
        r"""Selective optimization setting for this campaign, which includes a
        set of conversion actions to optimize this campaign towards. This
        feature only applies to app campaigns that use MULTI_CHANNEL as
        AdvertisingChannelType and APP_CAMPAIGN or
        APP_CAMPAIGN_FOR_ENGAGEMENT as AdvertisingChannelSubType.

        Attributes:
            conversion_actions (MutableSequence[str]):
                The selected set of resource names for
                conversion actions for optimizing this campaign.
        """

        conversion_actions: MutableSequence[str] = proto.RepeatedField(
            proto.STRING,
            number=2,
        )

    class OptimizationGoalSetting(proto.Message):
        r"""Optimization goal setting for this campaign, which includes a
        set of optimization goal types.

        Attributes:
            optimization_goal_types (MutableSequence[google.ads.googleads.v20.enums.types.OptimizationGoalTypeEnum.OptimizationGoalType]):
                The list of optimization goal types.
        """

        optimization_goal_types: MutableSequence[
            optimization_goal_type.OptimizationGoalTypeEnum.OptimizationGoalType
        ] = proto.RepeatedField(
            proto.ENUM,
            number=1,
            enum=optimization_goal_type.OptimizationGoalTypeEnum.OptimizationGoalType,
        )

    class AudienceSetting(proto.Message):
        r"""Settings for the audience targeting.

        .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

        Attributes:
            use_audience_grouped (bool):
                Immutable. If true, this campaign uses an
                Audience resource for audience targeting. If
                false, this campaign may use audience segment
                criteria instead.

                This field is a member of `oneof`_ ``_use_audience_grouped``.
        """

        use_audience_grouped: bool = proto.Field(
            proto.BOOL,
            number=1,
            optional=True,
        )

    class LocalServicesCampaignSettings(proto.Message):
        r"""Settings for LocalServicesCampaign subresource.

        Attributes:
            category_bids (MutableSequence[google.ads.googleads.v20.resources.types.Campaign.CategoryBid]):
                Categorical level bids associated with MANUAL_CPA bidding
                strategy.
        """

        category_bids: MutableSequence["Campaign.CategoryBid"] = (
            proto.RepeatedField(
                proto.MESSAGE,
                number=1,
                message="Campaign.CategoryBid",
            )
        )

    class CategoryBid(proto.Message):
        r"""Category bids in LocalServicesReportingCampaignSettings.

        .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

        Attributes:
            category_id (str):
                Category for which the bid will be associated with. For
                example, xcat:service_area_business_plumber.

                This field is a member of `oneof`_ ``_category_id``.
            manual_cpa_bid_micros (int):
                Manual CPA bid for the category. Bid must be
                greater than the reserve price associated for
                that category. Value is in micros and in the
                advertiser's currency.

                This field is a member of `oneof`_ ``_manual_cpa_bid_micros``.
            target_cpa_bid_micros (int):
                Target CPA bid for the category. Value is in
                micros and in the advertiser's currency.

                This field is a member of `oneof`_ ``_target_cpa_bid_micros``.
        """

        category_id: str = proto.Field(
            proto.STRING,
            number=1,
            optional=True,
        )
        manual_cpa_bid_micros: int = proto.Field(
            proto.INT64,
            number=2,
            optional=True,
        )
        target_cpa_bid_micros: int = proto.Field(
            proto.INT64,
            number=3,
            optional=True,
        )

    class TravelCampaignSettings(proto.Message):
        r"""Settings for Travel campaign.

        .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

        Attributes:
            travel_account_id (int):
                Immutable. The Travel account ID associated
                with the Travel campaign.

                This field is a member of `oneof`_ ``_travel_account_id``.
        """

        travel_account_id: int = proto.Field(
            proto.INT64,
            number=1,
            optional=True,
        )

    class DemandGenCampaignSettings(proto.Message):
        r"""Settings for Demand Gen campaign.

        .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

        Attributes:
            upgraded_targeting (bool):
                Immutable. Specifies whether this campaign uses upgraded
                targeting options. When this field is set to ``true``, you
                can use location and language targeting at the ad group
                level as opposed to the standard campaign-level targeting.
                This field defaults to ``true``, and can only be set when
                creating a campaign.

                This field is a member of `oneof`_ ``_upgraded_targeting``.
        """

        upgraded_targeting: bool = proto.Field(
            proto.BOOL,
            number=1,
            optional=True,
        )

    class VideoCampaignSettings(proto.Message):
        r"""Settings for Video campaign.

        This message has `oneof`_ fields (mutually exclusive fields).
        For each oneof, at most one member field can be set at the same time.
        Setting any member of the oneof automatically clears all other
        members.

        .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

        Attributes:
            video_ad_inventory_control (google.ads.googleads.v20.resources.types.Campaign.VideoCampaignSettings.VideoAdInventoryControl):
                Inventory control for video responsive ads in
                reach campaigns.

                This field is a member of `oneof`_ ``fluidity_control``.
            video_ad_format_control (google.ads.googleads.v20.resources.types.Campaign.VideoCampaignSettings.VideoAdFormatControl):
                Format-restricting control enabling usage of
                video responsive ads in format defined Video
                campaigns (for example, non-skippable).

                This field is a member of `oneof`_ ``fluidity_control``.
        """

        class VideoAdInventoryControl(proto.Message):
            r"""For campaigns using video responsive ads inventory controls
            determine on which inventories the ads can be shown. This only
            applies for campaigns with the bidding strategies TARGET_CPM and
            FIXED_CPM.


            .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

            Attributes:
                allow_in_stream (bool):
                    Determine if video responsive ads can be used
                    for in-stream video ads.

                    This field is a member of `oneof`_ ``_allow_in_stream``.
                allow_in_feed (bool):
                    Determine if video responsive ads can be used
                    for in-feed video ads.

                    This field is a member of `oneof`_ ``_allow_in_feed``.
                allow_shorts (bool):
                    Determine if video responsive ads can be used
                    as shorts format.

                    This field is a member of `oneof`_ ``_allow_shorts``.
            """

            allow_in_stream: bool = proto.Field(
                proto.BOOL,
                number=1,
                optional=True,
            )
            allow_in_feed: bool = proto.Field(
                proto.BOOL,
                number=2,
                optional=True,
            )
            allow_shorts: bool = proto.Field(
                proto.BOOL,
                number=3,
                optional=True,
            )

        class VideoAdFormatControl(proto.Message):
            r"""Format-restricting control enabling usage of video responsive
            ads in format defined Video campaigns (for example,
            non-skippable).

            Attributes:
                format_restriction (google.ads.googleads.v20.enums.types.VideoAdFormatRestrictionEnum.VideoAdFormatRestriction):
                    All contained responsive ads are expected to
                    respect this restriction.
                non_skippable_in_stream_restrictions (google.ads.googleads.v20.resources.types.Campaign.VideoCampaignSettings.NonSkippableInStreamRestrictions):
                    Restrictions for non-skippable format.
            """

            format_restriction: (
                video_ad_format_restriction.VideoAdFormatRestrictionEnum.VideoAdFormatRestriction
            ) = proto.Field(
                proto.ENUM,
                number=1,
                enum=video_ad_format_restriction.VideoAdFormatRestrictionEnum.VideoAdFormatRestriction,
            )
            non_skippable_in_stream_restrictions: "Campaign.VideoCampaignSettings.NonSkippableInStreamRestrictions" = proto.Field(
                proto.MESSAGE,
                number=2,
                message="Campaign.VideoCampaignSettings.NonSkippableInStreamRestrictions",
            )

        class NonSkippableInStreamRestrictions(proto.Message):
            r"""Restrictions for non-skippable format.

            Attributes:
                min_duration (google.ads.googleads.v20.enums.types.NonSkippableMinDurationEnum.NonSkippableMinDuration):
                    The minimum allowed duration for
                    non-skippable ads.
                max_duration (google.ads.googleads.v20.enums.types.NonSkippableMaxDurationEnum.NonSkippableMaxDuration):
                    The maximum allowed duration for
                    non-skippable ads.
            """

            min_duration: (
                non_skippable_min_duration.NonSkippableMinDurationEnum.NonSkippableMinDuration
            ) = proto.Field(
                proto.ENUM,
                number=1,
                enum=non_skippable_min_duration.NonSkippableMinDurationEnum.NonSkippableMinDuration,
            )
            max_duration: (
                non_skippable_max_duration.NonSkippableMaxDurationEnum.NonSkippableMaxDuration
            ) = proto.Field(
                proto.ENUM,
                number=2,
                enum=non_skippable_max_duration.NonSkippableMaxDurationEnum.NonSkippableMaxDuration,
            )

        video_ad_inventory_control: (
            "Campaign.VideoCampaignSettings.VideoAdInventoryControl"
        ) = proto.Field(
            proto.MESSAGE,
            number=2,
            oneof="fluidity_control",
            message="Campaign.VideoCampaignSettings.VideoAdInventoryControl",
        )
        video_ad_format_control: (
            "Campaign.VideoCampaignSettings.VideoAdFormatControl"
        ) = proto.Field(
            proto.MESSAGE,
            number=3,
            oneof="fluidity_control",
            message="Campaign.VideoCampaignSettings.VideoAdFormatControl",
        )

    class PmaxCampaignSettings(proto.Message):
        r"""Settings for Performance Max campaigns.

        Attributes:
            brand_targeting_overrides (google.ads.googleads.v20.resources.types.Campaign.PmaxCampaignSettings.BrandTargetingOverrides):
                Overrides of brand targeting for various ad
                types.
        """

        class BrandTargetingOverrides(proto.Message):
            r"""Overrides of brand targeting for various ad types.

            .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

            Attributes:
                ignore_exclusions_for_shopping_ads (bool):
                    If true, brand exclusions are ignored for
                    Shopping ads.

                    This field is a member of `oneof`_ ``_ignore_exclusions_for_shopping_ads``.
            """

            ignore_exclusions_for_shopping_ads: bool = proto.Field(
                proto.BOOL,
                number=1,
                optional=True,
            )

        brand_targeting_overrides: (
            "Campaign.PmaxCampaignSettings.BrandTargetingOverrides"
        ) = proto.Field(
            proto.MESSAGE,
            number=1,
            message="Campaign.PmaxCampaignSettings.BrandTargetingOverrides",
        )

    class AssetAutomationSetting(proto.Message):
        r"""Asset automation setting contains pair of AssetAutomationType
        and the asset automation opt-in/out status


        .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

        Attributes:
            asset_automation_type (google.ads.googleads.v20.enums.types.AssetAutomationTypeEnum.AssetAutomationType):
                The asset automation type advertiser would
                like to opt-in/out.

                This field is a member of `oneof`_ ``_asset_automation_type``.
            asset_automation_status (google.ads.googleads.v20.enums.types.AssetAutomationStatusEnum.AssetAutomationStatus):
                The opt-in/out status of asset automation
                type.

                This field is a member of `oneof`_ ``_asset_automation_status``.
        """

        asset_automation_type: (
            gage_asset_automation_type.AssetAutomationTypeEnum.AssetAutomationType
        ) = proto.Field(
            proto.ENUM,
            number=1,
            optional=True,
            enum=gage_asset_automation_type.AssetAutomationTypeEnum.AssetAutomationType,
        )
        asset_automation_status: (
            gage_asset_automation_status.AssetAutomationStatusEnum.AssetAutomationStatus
        ) = proto.Field(
            proto.ENUM,
            number=2,
            optional=True,
            enum=gage_asset_automation_status.AssetAutomationStatusEnum.AssetAutomationStatus,
        )

    class BrandGuidelines(proto.Message):
        r"""Settings that control the visual appearance of your brand in
        a campaign's automatically generated assets and formats. Only
        applicable to Performance Max campaigns.

        Attributes:
            main_color (str):
                The main brand color, entered as a hex code (e.g., #00ff00).
                You must provide the main_color if you provide an
                accent_color.
            accent_color (str):
                The accent brand color, entered as a hex code (e.g.,
                #00ff00). You must provide the accent_color if you provide a
                main_color.
            predefined_font_family (str):
                The brand's font family. Must be one of the
                following Google Fonts (case sensitive): Open
                Sans, Roboto, Montserrat, Poppins, Lato, Oswald,
                Playfair Display, Roboto Slab.
        """

        main_color: str = proto.Field(
            proto.STRING,
            number=1,
        )
        accent_color: str = proto.Field(
            proto.STRING,
            number=2,
        )
        predefined_font_family: str = proto.Field(
            proto.STRING,
            number=3,
        )

    resource_name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    id: int = proto.Field(
        proto.INT64,
        number=59,
        optional=True,
    )
    name: str = proto.Field(
        proto.STRING,
        number=58,
        optional=True,
    )
    primary_status: (
        campaign_primary_status.CampaignPrimaryStatusEnum.CampaignPrimaryStatus
    ) = proto.Field(
        proto.ENUM,
        number=81,
        enum=campaign_primary_status.CampaignPrimaryStatusEnum.CampaignPrimaryStatus,
    )
    primary_status_reasons: MutableSequence[
        campaign_primary_status_reason.CampaignPrimaryStatusReasonEnum.CampaignPrimaryStatusReason
    ] = proto.RepeatedField(
        proto.ENUM,
        number=82,
        enum=campaign_primary_status_reason.CampaignPrimaryStatusReasonEnum.CampaignPrimaryStatusReason,
    )
    status: campaign_status.CampaignStatusEnum.CampaignStatus = proto.Field(
        proto.ENUM,
        number=5,
        enum=campaign_status.CampaignStatusEnum.CampaignStatus,
    )
    serving_status: (
        campaign_serving_status.CampaignServingStatusEnum.CampaignServingStatus
    ) = proto.Field(
        proto.ENUM,
        number=21,
        enum=campaign_serving_status.CampaignServingStatusEnum.CampaignServingStatus,
    )
    bidding_strategy_system_status: (
        gage_bidding_strategy_system_status.BiddingStrategySystemStatusEnum.BiddingStrategySystemStatus
    ) = proto.Field(
        proto.ENUM,
        number=78,
        enum=gage_bidding_strategy_system_status.BiddingStrategySystemStatusEnum.BiddingStrategySystemStatus,
    )
    ad_serving_optimization_status: (
        gage_ad_serving_optimization_status.AdServingOptimizationStatusEnum.AdServingOptimizationStatus
    ) = proto.Field(
        proto.ENUM,
        number=8,
        enum=gage_ad_serving_optimization_status.AdServingOptimizationStatusEnum.AdServingOptimizationStatus,
    )
    advertising_channel_type: (
        gage_advertising_channel_type.AdvertisingChannelTypeEnum.AdvertisingChannelType
    ) = proto.Field(
        proto.ENUM,
        number=9,
        enum=gage_advertising_channel_type.AdvertisingChannelTypeEnum.AdvertisingChannelType,
    )
    advertising_channel_sub_type: (
        gage_advertising_channel_sub_type.AdvertisingChannelSubTypeEnum.AdvertisingChannelSubType
    ) = proto.Field(
        proto.ENUM,
        number=10,
        enum=gage_advertising_channel_sub_type.AdvertisingChannelSubTypeEnum.AdvertisingChannelSubType,
    )
    tracking_url_template: str = proto.Field(
        proto.STRING,
        number=60,
        optional=True,
    )
    url_custom_parameters: MutableSequence[custom_parameter.CustomParameter] = (
        proto.RepeatedField(
            proto.MESSAGE,
            number=12,
            message=custom_parameter.CustomParameter,
        )
    )
    local_services_campaign_settings: LocalServicesCampaignSettings = (
        proto.Field(
            proto.MESSAGE,
            number=75,
            message=LocalServicesCampaignSettings,
        )
    )
    travel_campaign_settings: TravelCampaignSettings = proto.Field(
        proto.MESSAGE,
        number=85,
        message=TravelCampaignSettings,
    )
    demand_gen_campaign_settings: DemandGenCampaignSettings = proto.Field(
        proto.MESSAGE,
        number=91,
        message=DemandGenCampaignSettings,
    )
    video_campaign_settings: VideoCampaignSettings = proto.Field(
        proto.MESSAGE,
        number=94,
        message=VideoCampaignSettings,
    )
    pmax_campaign_settings: PmaxCampaignSettings = proto.Field(
        proto.MESSAGE,
        number=97,
        message=PmaxCampaignSettings,
    )
    real_time_bidding_setting: (
        gagc_real_time_bidding_setting.RealTimeBiddingSetting
    ) = proto.Field(
        proto.MESSAGE,
        number=39,
        message=gagc_real_time_bidding_setting.RealTimeBiddingSetting,
    )
    network_settings: NetworkSettings = proto.Field(
        proto.MESSAGE,
        number=14,
        message=NetworkSettings,
    )
    hotel_setting: HotelSettingInfo = proto.Field(
        proto.MESSAGE,
        number=32,
        message=HotelSettingInfo,
    )
    dynamic_search_ads_setting: DynamicSearchAdsSetting = proto.Field(
        proto.MESSAGE,
        number=33,
        message=DynamicSearchAdsSetting,
    )
    shopping_setting: ShoppingSetting = proto.Field(
        proto.MESSAGE,
        number=36,
        message=ShoppingSetting,
    )
    targeting_setting: gagc_targeting_setting.TargetingSetting = proto.Field(
        proto.MESSAGE,
        number=43,
        message=gagc_targeting_setting.TargetingSetting,
    )
    audience_setting: AudienceSetting = proto.Field(
        proto.MESSAGE,
        number=73,
        optional=True,
        message=AudienceSetting,
    )
    geo_target_type_setting: GeoTargetTypeSetting = proto.Field(
        proto.MESSAGE,
        number=47,
        message=GeoTargetTypeSetting,
    )
    local_campaign_setting: LocalCampaignSetting = proto.Field(
        proto.MESSAGE,
        number=50,
        message=LocalCampaignSetting,
    )
    app_campaign_setting: AppCampaignSetting = proto.Field(
        proto.MESSAGE,
        number=51,
        message=AppCampaignSetting,
    )
    labels: MutableSequence[str] = proto.RepeatedField(
        proto.STRING,
        number=61,
    )
    experiment_type: (
        campaign_experiment_type.CampaignExperimentTypeEnum.CampaignExperimentType
    ) = proto.Field(
        proto.ENUM,
        number=17,
        enum=campaign_experiment_type.CampaignExperimentTypeEnum.CampaignExperimentType,
    )
    base_campaign: str = proto.Field(
        proto.STRING,
        number=56,
        optional=True,
    )
    campaign_budget: str = proto.Field(
        proto.STRING,
        number=62,
        optional=True,
    )
    bidding_strategy_type: (
        gage_bidding_strategy_type.BiddingStrategyTypeEnum.BiddingStrategyType
    ) = proto.Field(
        proto.ENUM,
        number=22,
        enum=gage_bidding_strategy_type.BiddingStrategyTypeEnum.BiddingStrategyType,
    )
    accessible_bidding_strategy: str = proto.Field(
        proto.STRING,
        number=71,
    )
    start_date: str = proto.Field(
        proto.STRING,
        number=63,
        optional=True,
    )
    campaign_group: str = proto.Field(
        proto.STRING,
        number=76,
        optional=True,
    )
    end_date: str = proto.Field(
        proto.STRING,
        number=64,
        optional=True,
    )
    final_url_suffix: str = proto.Field(
        proto.STRING,
        number=65,
        optional=True,
    )
    frequency_caps: MutableSequence[frequency_cap.FrequencyCapEntry] = (
        proto.RepeatedField(
            proto.MESSAGE,
            number=40,
            message=frequency_cap.FrequencyCapEntry,
        )
    )
    video_brand_safety_suitability: (
        brand_safety_suitability.BrandSafetySuitabilityEnum.BrandSafetySuitability
    ) = proto.Field(
        proto.ENUM,
        number=42,
        enum=brand_safety_suitability.BrandSafetySuitabilityEnum.BrandSafetySuitability,
    )
    vanity_pharma: VanityPharma = proto.Field(
        proto.MESSAGE,
        number=44,
        message=VanityPharma,
    )
    selective_optimization: SelectiveOptimization = proto.Field(
        proto.MESSAGE,
        number=45,
        message=SelectiveOptimization,
    )
    optimization_goal_setting: OptimizationGoalSetting = proto.Field(
        proto.MESSAGE,
        number=54,
        message=OptimizationGoalSetting,
    )
    tracking_setting: TrackingSetting = proto.Field(
        proto.MESSAGE,
        number=46,
        message=TrackingSetting,
    )
    payment_mode: gage_payment_mode.PaymentModeEnum.PaymentMode = proto.Field(
        proto.ENUM,
        number=52,
        enum=gage_payment_mode.PaymentModeEnum.PaymentMode,
    )
    optimization_score: float = proto.Field(
        proto.DOUBLE,
        number=66,
        optional=True,
    )
    excluded_parent_asset_field_types: MutableSequence[
        asset_field_type.AssetFieldTypeEnum.AssetFieldType
    ] = proto.RepeatedField(
        proto.ENUM,
        number=69,
        enum=asset_field_type.AssetFieldTypeEnum.AssetFieldType,
    )
    excluded_parent_asset_set_types: MutableSequence[
        asset_set_type.AssetSetTypeEnum.AssetSetType
    ] = proto.RepeatedField(
        proto.ENUM,
        number=80,
        enum=asset_set_type.AssetSetTypeEnum.AssetSetType,
    )
    url_expansion_opt_out: bool = proto.Field(
        proto.BOOL,
        number=72,
        optional=True,
    )
    performance_max_upgrade: PerformanceMaxUpgrade = proto.Field(
        proto.MESSAGE,
        number=77,
        message=PerformanceMaxUpgrade,
    )
    hotel_property_asset_set: str = proto.Field(
        proto.STRING,
        number=83,
        optional=True,
    )
    listing_type: gage_listing_type.ListingTypeEnum.ListingType = proto.Field(
        proto.ENUM,
        number=86,
        optional=True,
        enum=gage_listing_type.ListingTypeEnum.ListingType,
    )
    asset_automation_settings: MutableSequence[AssetAutomationSetting] = (
        proto.RepeatedField(
            proto.MESSAGE,
            number=88,
            message=AssetAutomationSetting,
        )
    )
    keyword_match_type: (
        campaign_keyword_match_type.CampaignKeywordMatchTypeEnum.CampaignKeywordMatchType
    ) = proto.Field(
        proto.ENUM,
        number=90,
        enum=campaign_keyword_match_type.CampaignKeywordMatchTypeEnum.CampaignKeywordMatchType,
    )
    brand_guidelines_enabled: bool = proto.Field(
        proto.BOOL,
        number=96,
        optional=True,
    )
    brand_guidelines: BrandGuidelines = proto.Field(
        proto.MESSAGE,
        number=98,
        message=BrandGuidelines,
    )
    contains_eu_political_advertising: (
        eu_political_advertising_status.EuPoliticalAdvertisingStatusEnum.EuPoliticalAdvertisingStatus
    ) = proto.Field(
        proto.ENUM,
        number=102,
        enum=eu_political_advertising_status.EuPoliticalAdvertisingStatusEnum.EuPoliticalAdvertisingStatus,
    )
    bidding_strategy: str = proto.Field(
        proto.STRING,
        number=67,
        oneof="campaign_bidding_strategy",
    )
    commission: bidding.Commission = proto.Field(
        proto.MESSAGE,
        number=49,
        oneof="campaign_bidding_strategy",
        message=bidding.Commission,
    )
    manual_cpa: bidding.ManualCpa = proto.Field(
        proto.MESSAGE,
        number=74,
        oneof="campaign_bidding_strategy",
        message=bidding.ManualCpa,
    )
    manual_cpc: bidding.ManualCpc = proto.Field(
        proto.MESSAGE,
        number=24,
        oneof="campaign_bidding_strategy",
        message=bidding.ManualCpc,
    )
    manual_cpm: bidding.ManualCpm = proto.Field(
        proto.MESSAGE,
        number=25,
        oneof="campaign_bidding_strategy",
        message=bidding.ManualCpm,
    )
    manual_cpv: bidding.ManualCpv = proto.Field(
        proto.MESSAGE,
        number=37,
        oneof="campaign_bidding_strategy",
        message=bidding.ManualCpv,
    )
    maximize_conversions: bidding.MaximizeConversions = proto.Field(
        proto.MESSAGE,
        number=30,
        oneof="campaign_bidding_strategy",
        message=bidding.MaximizeConversions,
    )
    maximize_conversion_value: bidding.MaximizeConversionValue = proto.Field(
        proto.MESSAGE,
        number=31,
        oneof="campaign_bidding_strategy",
        message=bidding.MaximizeConversionValue,
    )
    target_cpa: bidding.TargetCpa = proto.Field(
        proto.MESSAGE,
        number=26,
        oneof="campaign_bidding_strategy",
        message=bidding.TargetCpa,
    )
    target_impression_share: bidding.TargetImpressionShare = proto.Field(
        proto.MESSAGE,
        number=48,
        oneof="campaign_bidding_strategy",
        message=bidding.TargetImpressionShare,
    )
    target_roas: bidding.TargetRoas = proto.Field(
        proto.MESSAGE,
        number=29,
        oneof="campaign_bidding_strategy",
        message=bidding.TargetRoas,
    )
    target_spend: bidding.TargetSpend = proto.Field(
        proto.MESSAGE,
        number=27,
        oneof="campaign_bidding_strategy",
        message=bidding.TargetSpend,
    )
    percent_cpc: bidding.PercentCpc = proto.Field(
        proto.MESSAGE,
        number=34,
        oneof="campaign_bidding_strategy",
        message=bidding.PercentCpc,
    )
    target_cpm: bidding.TargetCpm = proto.Field(
        proto.MESSAGE,
        number=41,
        oneof="campaign_bidding_strategy",
        message=bidding.TargetCpm,
    )
    fixed_cpm: bidding.FixedCpm = proto.Field(
        proto.MESSAGE,
        number=92,
        oneof="campaign_bidding_strategy",
        message=bidding.FixedCpm,
    )
    target_cpv: bidding.TargetCpv = proto.Field(
        proto.MESSAGE,
        number=93,
        oneof="campaign_bidding_strategy",
        message=bidding.TargetCpv,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations

from typing import MutableSequence

import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v19.common",
    marshal="google.ads.googleads.v19",
    manifest={
        "CpcBidSimulationPointList",
        "CpvBidSimulationPointList",
        "TargetCpaSimulationPointList",
        "TargetRoasSimulationPointList",
        "PercentCpcBidSimulationPointList",
        "BudgetSimulationPointList",
        "TargetImpressionShareSimulationPointList",
        "CpcBidSimulationPoint",
        "CpvBidSimulationPoint",
        "TargetCpaSimulationPoint",
        "TargetRoasSimulationPoint",
        "PercentCpcBidSimulationPoint",
        "BudgetSimulationPoint",
        "TargetImpressionShareSimulationPoint",
    },
)


class CpcBidSimulationPointList(proto.Message):
    r"""A container for simulation points for simulations of type CPC_BID.

    Attributes:
        points (MutableSequence[google.ads.googleads.v19.common.types.CpcBidSimulationPoint]):
            Projected metrics for a series of CPC bid
            amounts.
    """

    points: MutableSequence["CpcBidSimulationPoint"] = proto.RepeatedField(
        proto.MESSAGE,
        number=1,
        message="CpcBidSimulationPoint",
    )


class CpvBidSimulationPointList(proto.Message):
    r"""A container for simulation points for simulations of type CPV_BID.

    Attributes:
        points (MutableSequence[google.ads.googleads.v19.common.types.CpvBidSimulationPoint]):
            Projected metrics for a series of CPV bid
            amounts.
    """

    points: MutableSequence["CpvBidSimulationPoint"] = proto.RepeatedField(
        proto.MESSAGE,
        number=1,
        message="CpvBidSimulationPoint",
    )


class TargetCpaSimulationPointList(proto.Message):
    r"""A container for simulation points for simulations of type
    TARGET_CPA.

    Attributes:
        points (MutableSequence[google.ads.googleads.v19.common.types.TargetCpaSimulationPoint]):
            Projected metrics for a series of target CPA
            amounts.
    """

    points: MutableSequence["TargetCpaSimulationPoint"] = proto.RepeatedField(
        proto.MESSAGE,
        number=1,
        message="TargetCpaSimulationPoint",
    )


class TargetRoasSimulationPointList(proto.Message):
    r"""A container for simulation points for simulations of type
    TARGET_ROAS.

    Attributes:
        points (MutableSequence[google.ads.googleads.v19.common.types.TargetRoasSimulationPoint]):
            Projected metrics for a series of target ROAS
            amounts.
    """

    points: MutableSequence["TargetRoasSimulationPoint"] = proto.RepeatedField(
        proto.MESSAGE,
        number=1,
        message="TargetRoasSimulationPoint",
    )


class PercentCpcBidSimulationPointList(proto.Message):
    r"""A container for simulation points for simulations of type
    PERCENT_CPC_BID.

    Attributes:
        points (MutableSequence[google.ads.googleads.v19.common.types.PercentCpcBidSimulationPoint]):
            Projected metrics for a series of percent CPC
            bid amounts.
    """

    points: MutableSequence["PercentCpcBidSimulationPoint"] = (
        proto.RepeatedField(
            proto.MESSAGE,
            number=1,
            message="PercentCpcBidSimulationPoint",
        )
    )


class BudgetSimulationPointList(proto.Message):
    r"""A container for simulation points for simulations of type
    BUDGET.

    Attributes:
        points (MutableSequence[google.ads.googleads.v19.common.types.BudgetSimulationPoint]):
            Projected metrics for a series of budget
            amounts.
    """

    points: MutableSequence["BudgetSimulationPoint"] = proto.RepeatedField(
        proto.MESSAGE,
        number=1,
        message="BudgetSimulationPoint",
    )


class TargetImpressionShareSimulationPointList(proto.Message):
    r"""A container for simulation points for simulations of type
    TARGET_IMPRESSION_SHARE.

    Attributes:
        points (MutableSequence[google.ads.googleads.v19.common.types.TargetImpressionShareSimulationPoint]):
            Projected metrics for a specific target
            impression share value.
    """

    points: MutableSequence["TargetImpressionShareSimulationPoint"] = (
        proto.RepeatedField(
            proto.MESSAGE,
            number=1,
            message="TargetImpressionShareSimulationPoint",
        )
    )


class CpcBidSimulationPoint(proto.Message):
    r"""Projected metrics for a specific CPC bid amount.

    This message has `oneof`_ fields (mutually exclusive fields).
    For each oneof, at most one member field can be set at the same time.
    Setting any member of the oneof automatically clears all other
    members.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        required_budget_amount_micros (int):
            Projected required daily budget that the
            advertiser must set in order to receive the
            estimated traffic, in micros of advertiser
            currency.
        biddable_conversions (float):
            Projected number of biddable conversions.

            This field is a member of `oneof`_ ``_biddable_conversions``.
        biddable_conversions_value (float):
            Projected total value of biddable
            conversions.

            This field is a member of `oneof`_ ``_biddable_conversions_value``.
        clicks (int):
            Projected number of clicks.

            This field is a member of `oneof`_ ``_clicks``.
        cost_micros (int):
            Projected cost in micros.

            This field is a member of `oneof`_ ``_cost_micros``.
        impressions (int):
            Projected number of impressions.

            This field is a member of `oneof`_ ``_impressions``.
        top_slot_impressions (int):
            Projected number of top slot impressions.
            Only search advertising channel type supports
            this field.

            This field is a member of `oneof`_ ``_top_slot_impressions``.
        cpc_bid_micros (int):
            The simulated CPC bid upon which projected
            metrics are based.

            This field is a member of `oneof`_ ``cpc_simulation_key_value``.
        cpc_bid_scaling_modifier (float):
            The simulated scaling modifier upon which
            projected metrics are based. All CPC bids
            relevant to the simulated entity are scaled by
            this modifier.

            This field is a member of `oneof`_ ``cpc_simulation_key_value``.
    """

    required_budget_amount_micros: int = proto.Field(
        proto.INT64,
        number=17,
    )
    biddable_conversions: float = proto.Field(
        proto.DOUBLE,
        number=9,
        optional=True,
    )
    biddable_conversions_value: float = proto.Field(
        proto.DOUBLE,
        number=10,
        optional=True,
    )
    clicks: int = proto.Field(
        proto.INT64,
        number=11,
        optional=True,
    )
    cost_micros: int = proto.Field(
        proto.INT64,
        number=12,
        optional=True,
    )
    impressions: int = proto.Field(
        proto.INT64,
        number=13,
        optional=True,
    )
    top_slot_impressions: int = proto.Field(
        proto.INT64,
        number=14,
        optional=True,
    )
    cpc_bid_micros: int = proto.Field(
        proto.INT64,
        number=15,
        oneof="cpc_simulation_key_value",
    )
    cpc_bid_scaling_modifier: float = proto.Field(
        proto.DOUBLE,
        number=16,
        oneof="cpc_simulation_key_value",
    )


class CpvBidSimulationPoint(proto.Message):
    r"""Projected metrics for a specific CPV bid amount.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        cpv_bid_micros (int):
            The simulated CPV bid upon which projected
            metrics are based.

            This field is a member of `oneof`_ ``_cpv_bid_micros``.
        cost_micros (int):
            Projected cost in micros.

            This field is a member of `oneof`_ ``_cost_micros``.
        impressions (int):
            Projected number of impressions.

            This field is a member of `oneof`_ ``_impressions``.
        views (int):
            Projected number of views.

            This field is a member of `oneof`_ ``_views``.
    """

    cpv_bid_micros: int = proto.Field(
        proto.INT64,
        number=5,
        optional=True,
    )
    cost_micros: int = proto.Field(
        proto.INT64,
        number=6,
        optional=True,
    )
    impressions: int = proto.Field(
        proto.INT64,
        number=7,
        optional=True,
    )
    views: int = proto.Field(
        proto.INT64,
        number=8,
        optional=True,
    )


class TargetCpaSimulationPoint(proto.Message):
    r"""Projected metrics for a specific target CPA amount.

    This message has `oneof`_ fields (mutually exclusive fields).
    For each oneof, at most one member field can be set at the same time.
    Setting any member of the oneof automatically clears all other
    members.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        required_budget_amount_micros (int):
            Projected required daily budget that the
            advertiser must set in order to receive the
            estimated traffic, in micros of advertiser
            currency.
        biddable_conversions (float):
            Projected number of biddable conversions.

            This field is a member of `oneof`_ ``_biddable_conversions``.
        biddable_conversions_value (float):
            Projected total value of biddable
            conversions.

            This field is a member of `oneof`_ ``_biddable_conversions_value``.
        app_installs (float):
            Projected number of app installs.
        in_app_actions (float):
            Projected number of in-app actions.
        clicks (int):
            Projected number of clicks.

            This field is a member of `oneof`_ ``_clicks``.
        cost_micros (int):
            Projected cost in micros.

            This field is a member of `oneof`_ ``_cost_micros``.
        impressions (int):
            Projected number of impressions.

            This field is a member of `oneof`_ ``_impressions``.
        top_slot_impressions (int):
            Projected number of top slot impressions.
            Only search advertising channel type supports
            this field.

            This field is a member of `oneof`_ ``_top_slot_impressions``.
        interactions (int):
            Projected number of interactions.
            Only discovery advertising channel type supports
            this field.

            This field is a member of `oneof`_ ``_interactions``.
        target_cpa_micros (int):
            The simulated target CPA upon which projected
            metrics are based.

            This field is a member of `oneof`_ ``target_cpa_simulation_key_value``.
        target_cpa_scaling_modifier (float):
            The simulated scaling modifier upon which
            projected metrics are based. All CPA targets
            relevant to the simulated entity are scaled by
            this modifier.

            This field is a member of `oneof`_ ``target_cpa_simulation_key_value``.
    """

    required_budget_amount_micros: int = proto.Field(
        proto.INT64,
        number=19,
    )
    biddable_conversions: float = proto.Field(
        proto.DOUBLE,
        number=9,
        optional=True,
    )
    biddable_conversions_value: float = proto.Field(
        proto.DOUBLE,
        number=10,
        optional=True,
    )
    app_installs: float = proto.Field(
        proto.DOUBLE,
        number=15,
    )
    in_app_actions: float = proto.Field(
        proto.DOUBLE,
        number=16,
    )
    clicks: int = proto.Field(
        proto.INT64,
        number=11,
        optional=True,
    )
    cost_micros: int = proto.Field(
        proto.INT64,
        number=12,
        optional=True,
    )
    impressions: int = proto.Field(
        proto.INT64,
        number=13,
        optional=True,
    )
    top_slot_impressions: int = proto.Field(
        proto.INT64,
        number=14,
        optional=True,
    )
    interactions: int = proto.Field(
        proto.INT64,
        number=20,
        optional=True,
    )
    target_cpa_micros: int = proto.Field(
        proto.INT64,
        number=17,
        oneof="target_cpa_simulation_key_value",
    )
    target_cpa_scaling_modifier: float = proto.Field(
        proto.DOUBLE,
        number=18,
        oneof="target_cpa_simulation_key_value",
    )


class TargetRoasSimulationPoint(proto.Message):
    r"""Projected metrics for a specific target ROAS amount.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        target_roas (float):
            The simulated target ROAS upon which
            projected metrics are based.

            This field is a member of `oneof`_ ``_target_roas``.
        required_budget_amount_micros (int):
            Projected required daily budget that the
            advertiser must set in order to receive the
            estimated traffic, in micros of advertiser
            currency.
        biddable_conversions (float):
            Projected number of biddable conversions.

            This field is a member of `oneof`_ ``_biddable_conversions``.
        biddable_conversions_value (float):
            Projected total value of biddable
            conversions.

            This field is a member of `oneof`_ ``_biddable_conversions_value``.
        clicks (int):
            Projected number of clicks.

            This field is a member of `oneof`_ ``_clicks``.
        cost_micros (int):
            Projected cost in micros.

            This field is a member of `oneof`_ ``_cost_micros``.
        impressions (int):
            Projected number of impressions.

            This field is a member of `oneof`_ ``_impressions``.
        top_slot_impressions (int):
            Projected number of top slot impressions.
            Only Search advertising channel type supports
            this field.

            This field is a member of `oneof`_ ``_top_slot_impressions``.
    """

    target_roas: float = proto.Field(
        proto.DOUBLE,
        number=8,
        optional=True,
    )
    required_budget_amount_micros: int = proto.Field(
        proto.INT64,
        number=15,
    )
    biddable_conversions: float = proto.Field(
        proto.DOUBLE,
        number=9,
        optional=True,
    )
    biddable_conversions_value: float = proto.Field(
        proto.DOUBLE,
        number=10,
        optional=True,
    )
    clicks: int = proto.Field(
        proto.INT64,
        number=11,
        optional=True,
    )
    cost_micros: int = proto.Field(
        proto.INT64,
        number=12,
        optional=True,
    )
    impressions: int = proto.Field(
        proto.INT64,
        number=13,
        optional=True,
    )
    top_slot_impressions: int = proto.Field(
        proto.INT64,
        number=14,
        optional=True,
    )


class PercentCpcBidSimulationPoint(proto.Message):
    r"""Projected metrics for a specific percent CPC amount. Only
    Hotel advertising channel type supports this field.


    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        percent_cpc_bid_micros (int):
            The simulated percent CPC upon which projected metrics are
            based. Percent CPC expressed as fraction of the advertised
            price for some good or service. The value stored here is
            1,000,000 \* [fraction].

            This field is a member of `oneof`_ ``_percent_cpc_bid_micros``.
        biddable_conversions (float):
            Projected number of biddable conversions.

            This field is a member of `oneof`_ ``_biddable_conversions``.
        biddable_conversions_value (float):
            Projected total value of biddable conversions
            in local currency.

            This field is a member of `oneof`_ ``_biddable_conversions_value``.
        clicks (int):
            Projected number of clicks.

            This field is a member of `oneof`_ ``_clicks``.
        cost_micros (int):
            Projected cost in micros.

            This field is a member of `oneof`_ ``_cost_micros``.
        impressions (int):
            Projected number of impressions.

            This field is a member of `oneof`_ ``_impressions``.
        top_slot_impressions (int):
            Projected number of top slot impressions.

            This field is a member of `oneof`_ ``_top_slot_impressions``.
    """

    percent_cpc_bid_micros: int = proto.Field(
        proto.INT64,
        number=1,
        optional=True,
    )
    biddable_conversions: float = proto.Field(
        proto.DOUBLE,
        number=2,
        optional=True,
    )
    biddable_conversions_value: float = proto.Field(
        proto.DOUBLE,
        number=3,
        optional=True,
    )
    clicks: int = proto.Field(
        proto.INT64,
        number=4,
        optional=True,
    )
    cost_micros: int = proto.Field(
        proto.INT64,
        number=5,
        optional=True,
    )
    impressions: int = proto.Field(
        proto.INT64,
        number=6,
        optional=True,
    )
    top_slot_impressions: int = proto.Field(
        proto.INT64,
        number=7,
        optional=True,
    )


class BudgetSimulationPoint(proto.Message):
    r"""Projected metrics for a specific budget amount.

    Attributes:
        budget_amount_micros (int):
            The simulated budget upon which projected
            metrics are based.
        required_cpc_bid_ceiling_micros (int):
            Projected required daily cpc bid ceiling that
            the advertiser must set to realize this
            simulation, in micros of the advertiser
            currency. Only campaigns with the Target Spend
            bidding strategy support this field.
        biddable_conversions (float):
            Projected number of biddable conversions.
        biddable_conversions_value (float):
            Projected total value of biddable
            conversions.
        clicks (int):
            Projected number of clicks.
        cost_micros (int):
            Projected cost in micros.
        impressions (int):
            Projected number of impressions.
        top_slot_impressions (int):
            Projected number of top slot impressions.
            Only search advertising channel type supports
            this field.
        interactions (int):
            Projected number of interactions.
            Only discovery advertising channel type supports
            this field.
    """

    budget_amount_micros: int = proto.Field(
        proto.INT64,
        number=1,
    )
    required_cpc_bid_ceiling_micros: int = proto.Field(
        proto.INT64,
        number=2,
    )
    biddable_conversions: float = proto.Field(
        proto.DOUBLE,
        number=3,
    )
    biddable_conversions_value: float = proto.Field(
        proto.DOUBLE,
        number=4,
    )
    clicks: int = proto.Field(
        proto.INT64,
        number=5,
    )
    cost_micros: int = proto.Field(
        proto.INT64,
        number=6,
    )
    impressions: int = proto.Field(
        proto.INT64,
        number=7,
    )
    top_slot_impressions: int = proto.Field(
        proto.INT64,
        number=8,
    )
    interactions: int = proto.Field(
        proto.INT64,
        number=9,
    )


class TargetImpressionShareSimulationPoint(proto.Message):
    r"""Projected metrics for a specific target impression share
    value.

    Attributes:
        target_impression_share_micros (int):
            The simulated target impression share value (in micros) upon
            which projected metrics are based. For example, 10%
            impression share, which is equal to 0.1, is stored as
            100_000. This value is validated and will not exceed 1M
            (100%).
        required_cpc_bid_ceiling_micros (int):
            Projected required daily cpc bid ceiling that
            the advertiser must set to realize this
            simulation, in micros of the advertiser
            currency.
        required_budget_amount_micros (int):
            Projected required daily budget that the
            advertiser must set in order to receive the
            estimated traffic, in micros of advertiser
            currency.
        biddable_conversions (float):
            Projected number of biddable conversions.
        biddable_conversions_value (float):
            Projected total value of biddable
            conversions.
        clicks (int):
            Projected number of clicks.
        cost_micros (int):
            Projected cost in micros.
        impressions (int):
            Projected number of impressions.
        top_slot_impressions (int):
            Projected number of top slot impressions.
            Only search advertising channel type supports
            this field.
        absolute_top_impressions (int):
            Projected number of absolute top impressions.
            Only search advertising channel type supports
            this field.
    """

    target_impression_share_micros: int = proto.Field(
        proto.INT64,
        number=1,
    )
    required_cpc_bid_ceiling_micros: int = proto.Field(
        proto.INT64,
        number=2,
    )
    required_budget_amount_micros: int = proto.Field(
        proto.INT64,
        number=3,
    )
    biddable_conversions: float = proto.Field(
        proto.DOUBLE,
        number=4,
    )
    biddable_conversions_value: float = proto.Field(
        proto.DOUBLE,
        number=5,
    )
    clicks: int = proto.Field(
        proto.INT64,
        number=6,
    )
    cost_micros: int = proto.Field(
        proto.INT64,
        number=7,
    )
    impressions: int = proto.Field(
        proto.INT64,
        number=8,
    )
    top_slot_impressions: int = proto.Field(
        proto.INT64,
        number=9,
    )
    absolute_top_impressions: int = proto.Field(
        proto.INT64,
        number=10,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations

from typing import MutableSequence

import proto  # type: ignore

from google.ads.googleads.v19.common.types import audience_insights_attribute
from google.ads.googleads.v19.common.types import criteria
from google.ads.googleads.v19.enums.types import insights_trend


__protobuf__ = proto.module(
    package="google.ads.googleads.v19.services",
    marshal="google.ads.googleads.v19",
    manifest={
        "GenerateCreatorInsightsRequest",
        "GenerateCreatorInsightsResponse",
        "GenerateTrendingInsightsRequest",
        "GenerateTrendingInsightsResponse",
        "YouTubeCreatorInsights",
        "YouTubeMetrics",
        "YouTubeChannelInsights",
        "SearchAudience",
        "SearchTopics",
        "TrendInsight",
        "TrendInsightMetrics",
    },
)


class GenerateCreatorInsightsRequest(proto.Message):
    r"""Request message for
    [ContentCreatorInsightsService.GenerateCreatorInsights][google.ads.googleads.v19.services.ContentCreatorInsightsService.GenerateCreatorInsights].

    This message has `oneof`_ fields (mutually exclusive fields).
    For each oneof, at most one member field can be set at the same time.
    Setting any member of the oneof automatically clears all other
    members.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        customer_id (str):
            Required. The ID of the customer.
        customer_insights_group (str):
            Required. The name of the customer being
            planned for.  This is a user-defined value.
        country_locations (MutableSequence[google.ads.googleads.v19.common.types.LocationInfo]):
            Required. The countries to search that apply
            to the criteria.
        search_attributes (google.ads.googleads.v19.services.types.GenerateCreatorInsightsRequest.SearchAttributes):
            The attributes used to identify top creators. Data fetched
            is based on the list of countries specified in
            [country_locations].

            This field is a member of `oneof`_ ``criteria``.
        search_brand (google.ads.googleads.v19.services.types.GenerateCreatorInsightsRequest.SearchBrand):
            A brand used to search for top creators. Data fetched is
            based on the list of countries specified in
            [country_locations].

            This field is a member of `oneof`_ ``criteria``.
        search_channels (google.ads.googleads.v19.services.types.GenerateCreatorInsightsRequest.YouTubeChannels):
            YouTube Channel IDs for Creator Insights. Data fetched for
            channels is based on the list of countries specified in
            [country_locations].

            This field is a member of `oneof`_ ``criteria``.
    """

    class SearchAttributes(proto.Message):
        r"""The audience attributes (such as Age, Gender, Affinity, and
        In-Market) and creator attributes (such as creator's content
        topics) used to search for top creators.

        Attributes:
            audience_attributes (MutableSequence[google.ads.googleads.v19.common.types.AudienceInsightsAttribute]):
                Optional. Audience attributes that describe an audience of
                viewers. This is used to search for creators whose own
                viewers match the input audience. Attributes age_range,
                gender, user_interest, entity, category, parental_status,
                and income_range are supported. Attribute location is not
                supported.
            creator_attributes (MutableSequence[google.ads.googleads.v19.common.types.AudienceInsightsAttribute]):
                Optional. Creator attributes that describe a collection of
                types of content. This is used to search for creators whose
                content matches the input creator attributes. Attribute
                entity tagged with
                [InsightsKnowledgeGraphEntityCapabilities.CREATOR_ATTRIBUTE][]
                is supported. Other attributes including location are not
                supported.
        """

        audience_attributes: MutableSequence[
            audience_insights_attribute.AudienceInsightsAttribute
        ] = proto.RepeatedField(
            proto.MESSAGE,
            number=1,
            message=audience_insights_attribute.AudienceInsightsAttribute,
        )
        creator_attributes: MutableSequence[
            audience_insights_attribute.AudienceInsightsAttribute
        ] = proto.RepeatedField(
            proto.MESSAGE,
            number=2,
            message=audience_insights_attribute.AudienceInsightsAttribute,
        )

    class SearchBrand(proto.Message):
        r"""The brand used to search for top creators.

        Attributes:
            brand_entities (MutableSequence[google.ads.googleads.v19.common.types.AudienceInsightsAttribute]):
                Optional. One or more Knowledge Graph
                Entities that represent the brand for which to
                find insights.
            include_related_topics (bool):
                Optional. When true, we will expand the search to beyond
                just the entities specified in [brand_entities] to other
                related knowledge graph entities similar to the brand. The
                default value is ``false``.
        """

        brand_entities: MutableSequence[
            audience_insights_attribute.AudienceInsightsAttribute
        ] = proto.RepeatedField(
            proto.MESSAGE,
            number=1,
            message=audience_insights_attribute.AudienceInsightsAttribute,
        )
        include_related_topics: bool = proto.Field(
            proto.BOOL,
            number=2,
        )

    class YouTubeChannels(proto.Message):
        r"""A collection of YouTube Channels.

        Attributes:
            youtube_channels (MutableSequence[google.ads.googleads.v19.common.types.YouTubeChannelInfo]):
                Optional. The YouTube Channel IDs to fetch
                creator insights for.
        """

        youtube_channels: MutableSequence[criteria.YouTubeChannelInfo] = (
            proto.RepeatedField(
                proto.MESSAGE,
                number=1,
                message=criteria.YouTubeChannelInfo,
            )
        )

    customer_id: str = proto.Field(
        proto.STRING,
        number=1,
    )
    customer_insights_group: str = proto.Field(
        proto.STRING,
        number=2,
    )
    country_locations: MutableSequence[criteria.LocationInfo] = (
        proto.RepeatedField(
            proto.MESSAGE,
            number=6,
            message=criteria.LocationInfo,
        )
    )
    search_attributes: SearchAttributes = proto.Field(
        proto.MESSAGE,
        number=3,
        oneof="criteria",
        message=SearchAttributes,
    )
    search_brand: SearchBrand = proto.Field(
        proto.MESSAGE,
        number=5,
        oneof="criteria",
        message=SearchBrand,
    )
    search_channels: YouTubeChannels = proto.Field(
        proto.MESSAGE,
        number=4,
        oneof="criteria",
        message=YouTubeChannels,
    )


class GenerateCreatorInsightsResponse(proto.Message):
    r"""Response message for
    [ContentCreatorInsightsService.GenerateCreatorInsights][google.ads.googleads.v19.services.ContentCreatorInsightsService.GenerateCreatorInsights].

    Attributes:
        creator_insights (MutableSequence[google.ads.googleads.v19.services.types.YouTubeCreatorInsights]):
            A collection of YouTube Creators, each
            containing a collection of YouTube Channels
            maintained by the YouTube Creator.
    """

    creator_insights: MutableSequence["YouTubeCreatorInsights"] = (
        proto.RepeatedField(
            proto.MESSAGE,
            number=1,
            message="YouTubeCreatorInsights",
        )
    )


class GenerateTrendingInsightsRequest(proto.Message):
    r"""Request message for
    [ContentCreatorInsightsService.GenerateTrendingInsights]

    This message has `oneof`_ fields (mutually exclusive fields).
    For each oneof, at most one member field can be set at the same time.
    Setting any member of the oneof automatically clears all other
    members.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        customer_id (str):
            Required. The ID of the customer.
        customer_insights_group (str):
            Required. The name of the customer being
            planned for. This is a user-defined value.
        country_location (google.ads.googleads.v19.common.types.LocationInfo):
            Required. The country to find trends in.
        search_audience (google.ads.googleads.v19.services.types.SearchAudience):
            An audience to search for trending content
            in.

            This field is a member of `oneof`_ ``criteria``.
        search_topics (google.ads.googleads.v19.services.types.SearchTopics):
            Content topics to return trend information
            for.

            This field is a member of `oneof`_ ``criteria``.
    """

    customer_id: str = proto.Field(
        proto.STRING,
        number=1,
    )
    customer_insights_group: str = proto.Field(
        proto.STRING,
        number=2,
    )
    country_location: criteria.LocationInfo = proto.Field(
        proto.MESSAGE,
        number=3,
        message=criteria.LocationInfo,
    )
    search_audience: "SearchAudience" = proto.Field(
        proto.MESSAGE,
        number=4,
        oneof="criteria",
        message="SearchAudience",
    )
    search_topics: "SearchTopics" = proto.Field(
        proto.MESSAGE,
        number=5,
        oneof="criteria",
        message="SearchTopics",
    )


class GenerateTrendingInsightsResponse(proto.Message):
    r"""Response message for
    [ContentCreatorInsightsService.GenerateTrendingInsights]

    Attributes:
        trend_insights (MutableSequence[google.ads.googleads.v19.services.types.TrendInsight]):
            The list of trending insights for the given
            criteria.
    """

    trend_insights: MutableSequence["TrendInsight"] = proto.RepeatedField(
        proto.MESSAGE,
        number=1,
        message="TrendInsight",
    )


class YouTubeCreatorInsights(proto.Message):
    r"""A YouTube creator and the insights for this creator.

    Attributes:
        creator_name (str):
            The name of the creator.
        creator_channels (MutableSequence[google.ads.googleads.v19.services.types.YouTubeChannelInsights]):
            The list of YouTube Channels
    """

    creator_name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    creator_channels: MutableSequence["YouTubeChannelInsights"] = (
        proto.RepeatedField(
            proto.MESSAGE,
            number=2,
            message="YouTubeChannelInsights",
        )
    )


class YouTubeMetrics(proto.Message):
    r"""YouTube Channel metrics.

    Attributes:
        subscriber_count (int):
            The number of subscribers.
        views_count (int):
            The total number of views.
        video_count (int):
            The total number of videos.
        is_active_shorts_creator (bool):
            When true, this channel has published a
            shorts video in the last 90 days.
    """

    subscriber_count: int = proto.Field(
        proto.INT64,
        number=1,
    )
    views_count: int = proto.Field(
        proto.INT64,
        number=2,
    )
    video_count: int = proto.Field(
        proto.INT64,
        number=3,
    )
    is_active_shorts_creator: bool = proto.Field(
        proto.BOOL,
        number=4,
    )


class YouTubeChannelInsights(proto.Message):
    r"""YouTube Channel insights, and its metadata (such as channel
    name and channel ID), returned for a creator insights response.

    Attributes:
        display_name (str):
            The name of the YouTube Channel.
        youtube_channel (google.ads.googleads.v19.common.types.YouTubeChannelInfo):
            The YouTube Channel ID.
        channel_url (str):
            URL for the channel in the form of
            https://www.youtube.com/channel/{channel_id}.
        channel_description (str):
            Description of the channel.
        channel_metrics (google.ads.googleads.v19.services.types.YouTubeMetrics):
            The metrics for a YouTube Channel.
        channel_audience_attributes (MutableSequence[google.ads.googleads.v19.common.types.AudienceInsightsAttributeMetadata]):
            The types of audiences and demographics
            linked to the channel's main audience. Audiences
            and demographics have a breakdown of subscriber
            share across dimensions of the same value, such
            as Age Range, Gender, and User Interest.
        channel_attributes (MutableSequence[google.ads.googleads.v19.common.types.AudienceInsightsAttributeMetadata]):
            The attributes associated with the content
            made by a channel.
        top_videos (MutableSequence[google.ads.googleads.v19.common.types.AudienceInsightsAttributeMetadata]):
            The top 10 videos for the channel.
        channel_type (str):
            Metadata string associated with the type of
            channel.
    """

    display_name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    youtube_channel: criteria.YouTubeChannelInfo = proto.Field(
        proto.MESSAGE,
        number=2,
        message=criteria.YouTubeChannelInfo,
    )
    channel_url: str = proto.Field(
        proto.STRING,
        number=9,
    )
    channel_description: str = proto.Field(
        proto.STRING,
        number=10,
    )
    channel_metrics: "YouTubeMetrics" = proto.Field(
        proto.MESSAGE,
        number=3,
        message="YouTubeMetrics",
    )
    channel_audience_attributes: MutableSequence[
        audience_insights_attribute.AudienceInsightsAttributeMetadata
    ] = proto.RepeatedField(
        proto.MESSAGE,
        number=7,
        message=audience_insights_attribute.AudienceInsightsAttributeMetadata,
    )
    channel_attributes: MutableSequence[
        audience_insights_attribute.AudienceInsightsAttributeMetadata
    ] = proto.RepeatedField(
        proto.MESSAGE,
        number=5,
        message=audience_insights_attribute.AudienceInsightsAttributeMetadata,
    )
    top_videos: MutableSequence[
        audience_insights_attribute.AudienceInsightsAttributeMetadata
    ] = proto.RepeatedField(
        proto.MESSAGE,
        number=8,
        message=audience_insights_attribute.AudienceInsightsAttributeMetadata,
    )
    channel_type: str = proto.Field(
        proto.STRING,
        number=6,
    )


class SearchAudience(proto.Message):
    r"""A collection of audience attributes that describe an audience
    of viewers. This is used to search for topics trending for the
    defined audience.

    Attributes:
        audience_attributes (MutableSequence[google.ads.googleads.v19.common.types.AudienceInsightsAttribute]):
            Required. Audience attributes that describe
            an audience of viewers. This is used to search
            for topics trending for the defined audience.
    """

    audience_attributes: MutableSequence[
        audience_insights_attribute.AudienceInsightsAttribute
    ] = proto.RepeatedField(
        proto.MESSAGE,
        number=1,
        message=audience_insights_attribute.AudienceInsightsAttribute,
    )


class SearchTopics(proto.Message):
    r"""A collection of content topics to return trend information
    for.

    Attributes:
        entities (MutableSequence[google.ads.googleads.v19.common.types.AudienceInsightsEntity]):
            Required. A list of knowledge graph entities to retrieve
            trend information for. Supported entities are tagged with
            [InsightsKnowledgeGraphEntityCapabilities.CONTENT_TRENDING_INSIGHTS][].
    """

    entities: MutableSequence[
        audience_insights_attribute.AudienceInsightsEntity
    ] = proto.RepeatedField(
        proto.MESSAGE,
        number=1,
        message=audience_insights_attribute.AudienceInsightsEntity,
    )


class TrendInsight(proto.Message):
    r"""A trend insight for a given attribute.

    Attributes:
        trend_attribute (google.ads.googleads.v19.common.types.AudienceInsightsAttributeMetadata):
            The attribute this trend is for.
        trend_metrics (google.ads.googleads.v19.services.types.TrendInsightMetrics):
            Metrics associated with this trend.
        trend (google.ads.googleads.v19.enums.types.InsightsTrendEnum.InsightsTrend):
            The direction of trend (such as RISING or
            DECLINING).
    """

    trend_attribute: (
        audience_insights_attribute.AudienceInsightsAttributeMetadata
    ) = proto.Field(
        proto.MESSAGE,
        number=1,
        message=audience_insights_attribute.AudienceInsightsAttributeMetadata,
    )
    trend_metrics: "TrendInsightMetrics" = proto.Field(
        proto.MESSAGE,
        number=2,
        message="TrendInsightMetrics",
    )
    trend: insights_trend.InsightsTrendEnum.InsightsTrend = proto.Field(
        proto.ENUM,
        number=3,
        enum=insights_trend.InsightsTrendEnum.InsightsTrend,
    )


class TrendInsightMetrics(proto.Message):
    r"""Metrics associated with a trend insight.

    Attributes:
        views_count (int):
            The number of views for this trend.
    """

    views_count: int = proto.Field(
        proto.INT64,
        number=1,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v20.enums",
    marshal="google.ads.googleads.v20",
    manifest={
        "PolicyTopicEntryTypeEnum",
    },
)


class PolicyTopicEntryTypeEnum(proto.Message):
    r"""Container for enum describing possible policy topic entry
    types.

    """

    class PolicyTopicEntryType(proto.Enum):
        r"""The possible policy topic entry types.

        Values:
            UNSPECIFIED (0):
                No value has been specified.
            UNKNOWN (1):
                The received value is not known in this
                version.
                This is a response-only value.
            PROHIBITED (2):
                The resource will not be served.
            LIMITED (4):
                The resource will not be served under some
                circumstances.
            FULLY_LIMITED (8):
                The resource cannot serve at all because of
                the current targeting criteria.
            DESCRIPTIVE (5):
                May be of interest, but does not limit how
                the resource is served.
            BROADENING (6):
                Could increase coverage beyond normal.
            AREA_OF_INTEREST_ONLY (7):
                Constrained for all targeted countries, but
                may serve in other countries through area of
                interest.
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        PROHIBITED = 2
        LIMITED = 4
        FULLY_LIMITED = 8
        DESCRIPTIVE = 5
        BROADENING = 6
        AREA_OF_INTEREST_ONLY = 7


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v20.errors",
    marshal="google.ads.googleads.v20",
    manifest={
        "FieldMaskErrorEnum",
    },
)


class FieldMaskErrorEnum(proto.Message):
    r"""Container for enum describing possible field mask errors."""

    class FieldMaskError(proto.Enum):
        r"""Enum describing possible field mask errors.

        Values:
            UNSPECIFIED (0):
                Enum unspecified.
            UNKNOWN (1):
                The received error code is not known in this
                version.
            FIELD_MASK_MISSING (5):
                The field mask must be provided for update
                operations.
            FIELD_MASK_NOT_ALLOWED (4):
                The field mask must be empty for create and
                remove operations.
            FIELD_NOT_FOUND (2):
                The field mask contained an invalid field.
            FIELD_HAS_SUBFIELDS (3):
                The field mask updated a field with
                subfields. Fields with subfields may be cleared,
                but not updated. To fix this, the field mask
                should select all the subfields of the invalid
                field.
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        FIELD_MASK_MISSING = 5
        FIELD_MASK_NOT_ALLOWED = 4
        FIELD_NOT_FOUND = 2
        FIELD_HAS_SUBFIELDS = 3


__all__ = tuple(sorted(__protobuf__.manifest))



#!/usr/bin/env python
# Copyright 2021 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Uploads an enhanced conversion for leads by uploading a ClickConversion.

The click conversion has hashed, first-party user-provided data from your
website lead forms. This includes user identifiers, and optionally, a click ID
and order ID. With this information, Google can tie the conversion to the ad
that drove the lead.
"""


import argparse
import hashlib
import re
import sys
from typing import Dict, Optional, Union

from google.ads.googleads.client import GoogleAdsClient
from google.ads.googleads.errors import GoogleAdsException
from google.ads.googleads.v22.common.types.offline_user_data import (
    UserIdentifier,
)
from google.ads.googleads.v22.services.services.conversion_action_service import (
    ConversionActionServiceClient,
)
from google.ads.googleads.v22.services.services.conversion_upload_service import (
    ConversionUploadServiceClient,
)
from google.ads.googleads.v22.services.types.conversion_upload_service import (
    ClickConversion,
    ClickConversionResult,
    SessionAttributeKeyValuePair,
    UploadClickConversionsResponse,
)


def main(
    client: GoogleAdsClient,
    customer_id: str,
    conversion_action_id: str,
    conversion_date_time: str,
    conversion_value: float,
    order_id: Optional[str],
    gclid: Optional[str],
    ad_user_data_consent: Optional[str],
    session_attributes_encoded: Optional[str] = None,
    session_attributes_dict: Optional[Dict[str, str]] = None,
) -> None:
    """The main method that creates all necessary entities for the example.

    Args:
        client: An initialized GoogleAdsClient instance.
        customer_id: The client customer ID string.
        conversion_action_id: The ID of the conversion action to upload to.
        conversion_date_time: The date and time of the conversion.
        conversion_value: The value of the conversion.
        order_id: The unique ID (transaction ID) of the conversion.
        gclid: The Google click ID for the click.
        ad_user_data_consent: The consent status for ad user data for all
            members in the job.
        session_attributes_encoded: a str token of encoded session atttributes.
            Only one of session_attributes_encoded or session_attributes_dict
            should be passed.
        session_attributes_dict: a dict[str, str] of session attribute
            key value pairs. Only one of session_attributes_encoded or
            session_attributes_dict should be passed.
    """
    if session_attributes_encoded and session_attributes_dict:
        raise ValueError(
            "Only one of 'session_attributes_encoded' or "
            "'session_attributes_dict' can be set."
        )

    # [START add_user_identifiers]
    # Extract user email and phone from the raw data, normalize and hash it,
    # then wrap it in UserIdentifier objects. Create a separate UserIdentifier
    # object for each. The data in this example is hardcoded, but in your
    # application you might read the raw data from an input file.

    # IMPORTANT: Since the identifier attribute of UserIdentifier
    # (https://developers.google.com/google-ads/api/reference/rpc/latest/UserIdentifier)
    # is a oneof
    # (https://protobuf.dev/programming-guides/proto3/#oneof-features), you must
    # set only ONE of hashed_email, hashed_phone_number, mobile_id,
    # third_party_user_id, or address_info. Setting more than one of these
    # attributes on the same UserIdentifier will clear all the other members of
    # the oneof. For example, the following code is INCORRECT and will result in
    # a UserIdentifier with ONLY a hashed_phone_number:
    #
    # incorrectly_populated_user_identifier = client.get_type("UserIdentifier")
    # incorrectly_populated_user_identifier.hashed_email = "...""
    # incorrectly_populated_user_identifier.hashed_phone_number = "...""

    raw_record: Dict[str, Union[str, float]] = {
        # Email address that includes a period (.) before the Gmail domain.
        "email": "alex.2@example.com",
        # Phone number to be converted to E.164 format, with a leading '+' as
        # required.
        "phone": "+1 800 5550102",
        # This example lets you input conversion details as arguments,
        # but in reality you might store this data alongside other user data,
        # so we include it in this sample user record.
        "order_id": order_id,
        "gclid": gclid,
        "conversion_action_id": conversion_action_id,
        "conversion_date_time": conversion_date_time,
        "conversion_value": conversion_value,
        "currency_code": "USD",
        "ad_user_data_consent": ad_user_data_consent,
    }

    # Constructs the click conversion.
    click_conversion: ClickConversion = client.get_type("ClickConversion")
    # Creates a user identifier using the hashed email address, using the
    # normalize and hash method specifically for email addresses.
    email_identifier: UserIdentifier = client.get_type("UserIdentifier")
    # Optional: Specifies the user identifier source.
    email_identifier.user_identifier_source = (
        client.enums.UserIdentifierSourceEnum.FIRST_PARTY
    )
    # Uses the normalize and hash method specifically for email addresses.
    email_identifier.hashed_email = normalize_and_hash_email_address(
        raw_record["email"]
    )
    # Adds the user identifier to the conversion.
    click_conversion.user_identifiers.append(email_identifier)

    # Checks if the record has a phone number, and if so, adds a UserIdentifier
    # for it.
    if raw_record.get("phone") is not None:
        phone_identifier: UserIdentifier = client.get_type("UserIdentifier")
        phone_identifier.hashed_phone_number = normalize_and_hash(
            raw_record["phone"]
        )
        # Adds the phone identifier to the conversion adjustment.
        click_conversion.user_identifiers.append(phone_identifier)
        # [END add_user_identifiers]

    # [START add_conversion_details]
    # Add details of the conversion.
    # Gets the conversion action resource name.
    conversion_action_service: ConversionActionServiceClient = (
        client.get_service("ConversionActionService")
    )
    click_conversion.conversion_action = (
        conversion_action_service.conversion_action_path(
            customer_id, raw_record["conversion_action_id"]
        )
    )
    click_conversion.conversion_date_time = raw_record["conversion_date_time"]
    click_conversion.conversion_value = raw_record["conversion_value"]
    click_conversion.currency_code = raw_record["currency_code"]

    # Sets the order ID if provided.
    if raw_record.get("order_id"):
        click_conversion.order_id = raw_record["order_id"]

    # Sets the gclid if provided.
    if raw_record.get("gclid"):
        click_conversion.gclid = raw_record["gclid"]

    # Specifies whether user consent was obtained for the data you are
    # uploading. For more details, see:
    # https://www.google.com/about/company/user-consent-policy
    if raw_record["ad_user_data_consent"]:
        click_conversion.consent.ad_user_data = client.enums.ConsentStatusEnum[
            raw_record["ad_user_data_consent"]
        ]

    # [START add_session_attributes]
    # Set one of the session_attributes_encoded or
    # session_attributes_key_value_pairs fields if either are provided.
    if session_attributes_encoded:
        click_conversion.session_attributes_encoded = session_attributes_encoded
    elif session_attributes_dict:
        for key, value in session_attributes_dict.items():
            pair: SessionAttributeKeyValuePair = client.get_type(
                "SessionAttributeKeyValuePair"
            )
            pair.session_attribute_key = key
            pair.session_attribute_value = value
            click_conversion.session_attributes_key_value_pairs.key_value_pairs.append(
                pair
            )
            # [END add_session_attributes]
            # [END add_conversion_details]

    # [START upload_conversion]
    # Creates the conversion upload service client.
    conversion_upload_service: ConversionUploadServiceClient = (
        client.get_service("ConversionUploadService")
    )
    # Uploads the click conversion. Partial failure should always be set to
    # True.
    # NOTE: This request only uploads a single conversion, but if you have
    # multiple conversions to upload, it's most efficient to upload them in a
    # single request. See the following for per-request limits for reference:
    # https://developers.google.com/google-ads/api/docs/best-practices/quotas#conversion_upload_service
    response: UploadClickConversionsResponse = (
        conversion_upload_service.upload_click_conversions(
            customer_id=customer_id,
            conversions=[click_conversion],
            # Enables partial failure (must be true).
            partial_failure=True,
        )
    )
    # [END upload_conversion]

    # Prints any partial errors returned.
    # To review the overall health of your recent uploads, see:
    # https://developers.google.com/google-ads/api/docs/conversions/upload-summaries
    if response.partial_failure_error:
        print(
            "Partial error encountered: "
            f"{response.partial_failure_error.message}"
        )
    else:
        # Prints the result.
        result: ClickConversionResult = response.results[0]
        print(
            "Uploaded conversion that occurred at "
            f"{result.conversion_date_time} "
            f"to {result.conversion_action}."
        )


# [START normalize_and_hash]
def normalize_and_hash_email_address(email_address: str) -> str:
    """Returns the result of normalizing and hashing an email address.

    For this use case, Google Ads requires removal of any '.' characters
    preceding "gmail.com" or "googlemail.com"

    Args:
        email_address: An email address to normalize.

    Returns:
        A normalized (lowercase, removed whitespace) and SHA-265 hashed string.
    """
    normalized_email: str = email_address.strip().lower()
    email_parts: list[str] = normalized_email.split("@")

    # Check that there are at least two segments
    if len(email_parts) > 1:
        # Checks whether the domain of the email address is either "gmail.com"
        # or "googlemail.com". If this regex does not match then this statement
        # will evaluate to None.
        if re.match(r"^(gmail|googlemail)\.com$", email_parts[1]):
            # Removes any '.' characters from the portion of the email address
            # before the domain if the domain is gmail.com or googlemail.com.
            email_parts[0] = email_parts[0].replace(".", "")
            normalized_email = "@".join(email_parts)

    return normalize_and_hash(normalized_email)


def normalize_and_hash(s: str) -> str:
    """Normalizes and hashes a string with SHA-256.

    Private customer data must be hashed during upload, as described at:
    https://support.google.com/google-ads/answer/7474263

    Args:
        s: The string to perform this operation on.

    Returns:
        A normalized (lowercase, removed whitespace) and SHA-256 hashed string.
    """
    return hashlib.sha256(s.strip().lower().encode()).hexdigest()
    # [END normalize_and_hash]


if __name__ == "__main__":
    # GoogleAdsClient will read the google-ads.yaml configuration file in the
    # home directory if none is specified.
    googleads_client: GoogleAdsClient = GoogleAdsClient.load_from_storage(
        version="v22"
    )

    parser: argparse.ArgumentParser = argparse.ArgumentParser(
        description="Imports offline call conversion values for calls related "
        "to your ads."
    )
    # The following argument(s) should be provided to run the example.
    parser.add_argument(
        "-c",
        "--customer_id",
        type=str,
        required=True,
        help="The Google Ads customer ID.",
    )
    parser.add_argument(
        "-a",
        "--conversion_action_id",
        type=str,
        required=True,
        help="The ID of the conversion action to upload to.",
    )
    parser.add_argument(
        "-d",
        "--conversion_date_time",
        type=str,
        required=True,
        help="The date time at which the conversion occurred. Must be after "
        "the click time, and must include the time zone offset.  The format is "
        "'yyyy-mm-dd hh:mm:ss+|-hh:mm', e.g. '2019-01-01 12:32:45-08:00'.",
    )
    parser.add_argument(
        "-v",
        "--conversion_value",
        type=float,
        required=True,
        help="The value of the conversion.",
    )
    parser.add_argument(
        "-o",
        "--order_id",
        type=str,
        help="the unique ID (transaction ID) of the conversion.",
    )
    parser.add_argument(
        "-g",
        "--gclid",
        type=str,
        help="the Google click ID (gclid) for the click.",
    )
    parser.add_argument(
        "-n",
        "--ad_user_data_consent",
        type=str,
        choices=[
            e.name
            for e in googleads_client.enums.ConsentStatusEnum
            if e.name not in ("UNSPECIFIED", "UNKNOWN")
        ],
        help=(
            "The data consent status for ad user data for all members in "
            "the job."
        ),
    )
    # A mutually exclusive group means that session_attributes_encoded and
    # session_attributes_key_value_pairs cannot be passed in at the same time.
    session_attributes = parser.add_mutually_exclusive_group()
    session_attributes.add_argument(
        "-e",
        "--session_attributes_encoded",
        type=str,
        default=None,
        help=("A session attributes token."),
    )
    session_attributes.add_argument(
        "-k",
        "--session_attributes_key_value_pairs",
        nargs="+",
        type=str,
        default=None,
        help=(
            "A space-delimited list of session attribute key value pairs. Each "
            "pair should be separated by an equal sign, for example: "
            "'-k gad_campaignid=12345 gad_source=1'"
        ),
    )
    args: argparse.Namespace = parser.parse_args()

    session_attributes_dict: Optional[Dict[str, str]] = None
    if args.session_attributes_key_value_pairs:
        # Convert the string-based input to a dict
        session_attributes_dict = dict(
            pair.split("=") for pair in args.session_attributes_key_value_pairs
        )

    try:
        main(
            googleads_client,
            args.customer_id,
            args.conversion_action_id,
            args.conversion_date_time,
            args.conversion_value,
            args.order_id,
            args.gclid,
            args.ad_user_data_consent,
            # Only one of 'session_attributes_encoded' or
            # 'session_attributes_dict' can be passed at a time. If both are
            # passed the example will fail with a ValueError.
            session_attributes_encoded=args.session_attributes_encoded,
            session_attributes_dict=session_attributes_dict,
        )
    except GoogleAdsException as ex:
        print(
            f"Request with ID '{ex.request_id}'' failed with status "
            f"'{ex.error.code().name}' and includes the following errors:"
        )
        for error in ex.failure.errors:
            print(f"\tError with message '{error.message}'.")
            if error.location:
                for field_path_element in error.location.field_path_elements:
                    print(f"\t\tOn field: {field_path_element.field_name}")
        sys.exit(1)



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations

from typing import MutableSequence

import proto  # type: ignore

from google.ads.googleads.v21.enums.types import google_ads_field_category
from google.ads.googleads.v21.enums.types import google_ads_field_data_type


__protobuf__ = proto.module(
    package="google.ads.googleads.v21.resources",
    marshal="google.ads.googleads.v21",
    manifest={
        "GoogleAdsField",
    },
)


class GoogleAdsField(proto.Message):
    r"""A field or resource (artifact) used by GoogleAdsService.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        resource_name (str):
            Output only. The resource name of the artifact. Artifact
            resource names have the form:

            ``googleAdsFields/{name}``
        name (str):
            Output only. The name of the artifact.

            This field is a member of `oneof`_ ``_name``.
        category (google.ads.googleads.v21.enums.types.GoogleAdsFieldCategoryEnum.GoogleAdsFieldCategory):
            Output only. The category of the artifact.
        selectable (bool):
            Output only. Whether the artifact can be used
            in a SELECT clause in search queries.

            This field is a member of `oneof`_ ``_selectable``.
        filterable (bool):
            Output only. Whether the artifact can be used
            in a WHERE clause in search queries.

            This field is a member of `oneof`_ ``_filterable``.
        sortable (bool):
            Output only. Whether the artifact can be used
            in a ORDER BY clause in search queries.

            This field is a member of `oneof`_ ``_sortable``.
        selectable_with (MutableSequence[str]):
            Output only. The names of all resources,
            segments, and metrics that are selectable with
            the described artifact.
        attribute_resources (MutableSequence[str]):
            Output only. The names of all resources that
            are selectable with the described artifact.
            Fields from these resources do not segment
            metrics when included in search queries.

            This field is only set for artifacts whose
            category is RESOURCE.
        metrics (MutableSequence[str]):
            Output only. This field lists the names of
            all metrics that are selectable with the
            described artifact when it is used in the FROM
            clause. It is only set for artifacts whose
            category is RESOURCE.
        segments (MutableSequence[str]):
            Output only. This field lists the names of
            all artifacts, whether a segment or another
            resource, that segment metrics when included in
            search queries and when the described artifact
            is used in the FROM clause. It is only set for
            artifacts whose category is RESOURCE.
        enum_values (MutableSequence[str]):
            Output only. Values the artifact can assume
            if it is a field of type ENUM.
            This field is only set for artifacts of category
            SEGMENT or ATTRIBUTE.
        data_type (google.ads.googleads.v21.enums.types.GoogleAdsFieldDataTypeEnum.GoogleAdsFieldDataType):
            Output only. This field determines the
            operators that can be used with the artifact in
            WHERE clauses.
        type_url (str):
            Output only. The URL of proto describing the
            artifact's data type.

            This field is a member of `oneof`_ ``_type_url``.
        is_repeated (bool):
            Output only. Whether the field artifact is
            repeated.

            This field is a member of `oneof`_ ``_is_repeated``.
    """

    resource_name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    name: str = proto.Field(
        proto.STRING,
        number=21,
        optional=True,
    )
    category: (
        google_ads_field_category.GoogleAdsFieldCategoryEnum.GoogleAdsFieldCategory
    ) = proto.Field(
        proto.ENUM,
        number=3,
        enum=google_ads_field_category.GoogleAdsFieldCategoryEnum.GoogleAdsFieldCategory,
    )
    selectable: bool = proto.Field(
        proto.BOOL,
        number=22,
        optional=True,
    )
    filterable: bool = proto.Field(
        proto.BOOL,
        number=23,
        optional=True,
    )
    sortable: bool = proto.Field(
        proto.BOOL,
        number=24,
        optional=True,
    )
    selectable_with: MutableSequence[str] = proto.RepeatedField(
        proto.STRING,
        number=25,
    )
    attribute_resources: MutableSequence[str] = proto.RepeatedField(
        proto.STRING,
        number=26,
    )
    metrics: MutableSequence[str] = proto.RepeatedField(
        proto.STRING,
        number=27,
    )
    segments: MutableSequence[str] = proto.RepeatedField(
        proto.STRING,
        number=28,
    )
    enum_values: MutableSequence[str] = proto.RepeatedField(
        proto.STRING,
        number=29,
    )
    data_type: (
        google_ads_field_data_type.GoogleAdsFieldDataTypeEnum.GoogleAdsFieldDataType
    ) = proto.Field(
        proto.ENUM,
        number=12,
        enum=google_ads_field_data_type.GoogleAdsFieldDataTypeEnum.GoogleAdsFieldDataType,
    )
    type_url: str = proto.Field(
        proto.STRING,
        number=30,
        optional=True,
    )
    is_repeated: bool = proto.Field(
        proto.BOOL,
        number=31,
        optional=True,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v21.enums",
    marshal="google.ads.googleads.v21",
    manifest={
        "SmartCampaignNotEligibleReasonEnum",
    },
)


class SmartCampaignNotEligibleReasonEnum(proto.Message):
    r"""A container for an enum that describes reasons for why a
    Smart campaign is not eligible to serve.

    """

    class SmartCampaignNotEligibleReason(proto.Enum):
        r"""Reasons for why a Smart campaign is not eligible to serve.

        Values:
            UNSPECIFIED (0):
                The status has not been specified.
            UNKNOWN (1):
                The received value is not known in this
                version.
                This is a response-only value.
            ACCOUNT_ISSUE (2):
                The campaign is not eligible to serve because
                of an issue with the account.
            BILLING_ISSUE (3):
                The campaign is not eligible to serve because
                of a payment issue.
            BUSINESS_PROFILE_LOCATION_REMOVED (4):
                The business profile location associated with
                the campaign has been removed.
            ALL_ADS_DISAPPROVED (5):
                All system-generated ads have been disapproved. Consult the
                policy_summary field in the AdGroupAd resource for more
                details.
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        ACCOUNT_ISSUE = 2
        BILLING_ISSUE = 3
        BUSINESS_PROFILE_LOCATION_REMOVED = 4
        ALL_ADS_DISAPPROVED = 5


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v21.enums",
    marshal="google.ads.googleads.v21",
    manifest={
        "ReachPlanAgeRangeEnum",
    },
)


class ReachPlanAgeRangeEnum(proto.Message):
    r"""Message describing plannable age ranges."""

    class ReachPlanAgeRange(proto.Enum):
        r"""Possible plannable age range values.

        Values:
            UNSPECIFIED (0):
                Not specified.
            UNKNOWN (1):
                The value is unknown in this version.
            AGE_RANGE_18_24 (503001):
                Between 18 and 24 years old.
            AGE_RANGE_18_34 (2):
                Between 18 and 34 years old.
            AGE_RANGE_18_44 (3):
                Between 18 and 44 years old.
            AGE_RANGE_18_49 (4):
                Between 18 and 49 years old.
            AGE_RANGE_18_54 (5):
                Between 18 and 54 years old.
            AGE_RANGE_18_64 (6):
                Between 18 and 64 years old.
            AGE_RANGE_18_65_UP (7):
                Between 18 and 65+ years old.
            AGE_RANGE_21_34 (8):
                Between 21 and 34 years old.
            AGE_RANGE_25_34 (503002):
                Between 25 and 34 years old.
            AGE_RANGE_25_44 (9):
                Between 25 and 44 years old.
            AGE_RANGE_25_49 (10):
                Between 25 and 49 years old.
            AGE_RANGE_25_54 (11):
                Between 25 and 54 years old.
            AGE_RANGE_25_64 (12):
                Between 25 and 64 years old.
            AGE_RANGE_25_65_UP (13):
                Between 25 and 65+ years old.
            AGE_RANGE_35_44 (503003):
                Between 35 and 44 years old.
            AGE_RANGE_35_49 (14):
                Between 35 and 49 years old.
            AGE_RANGE_35_54 (15):
                Between 35 and 54 years old.
            AGE_RANGE_35_64 (16):
                Between 35 and 64 years old.
            AGE_RANGE_35_65_UP (17):
                Between 35 and 65+ years old.
            AGE_RANGE_45_54 (503004):
                Between 45 and 54 years old.
            AGE_RANGE_45_64 (18):
                Between 45 and 64 years old.
            AGE_RANGE_45_65_UP (19):
                Between 45 and 65+ years old.
            AGE_RANGE_50_65_UP (20):
                Between 50 and 65+ years old.
            AGE_RANGE_55_64 (503005):
                Between 55 and 64 years old.
            AGE_RANGE_55_65_UP (21):
                Between 55 and 65+ years old.
            AGE_RANGE_65_UP (503006):
                65 years old and beyond.
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        AGE_RANGE_18_24 = 503001
        AGE_RANGE_18_34 = 2
        AGE_RANGE_18_44 = 3
        AGE_RANGE_18_49 = 4
        AGE_RANGE_18_54 = 5
        AGE_RANGE_18_64 = 6
        AGE_RANGE_18_65_UP = 7
        AGE_RANGE_21_34 = 8
        AGE_RANGE_25_34 = 503002
        AGE_RANGE_25_44 = 9
        AGE_RANGE_25_49 = 10
        AGE_RANGE_25_54 = 11
        AGE_RANGE_25_64 = 12
        AGE_RANGE_25_65_UP = 13
        AGE_RANGE_35_44 = 503003
        AGE_RANGE_35_49 = 14
        AGE_RANGE_35_54 = 15
        AGE_RANGE_35_64 = 16
        AGE_RANGE_35_65_UP = 17
        AGE_RANGE_45_54 = 503004
        AGE_RANGE_45_64 = 18
        AGE_RANGE_45_65_UP = 19
        AGE_RANGE_50_65_UP = 20
        AGE_RANGE_55_64 = 503005
        AGE_RANGE_55_65_UP = 21
        AGE_RANGE_65_UP = 503006


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore

from google.ads.googleads.v19.enums.types import shared_set_status
from google.ads.googleads.v19.enums.types import shared_set_type


__protobuf__ = proto.module(
    package="google.ads.googleads.v19.resources",
    marshal="google.ads.googleads.v19",
    manifest={
        "SharedSet",
    },
)


class SharedSet(proto.Message):
    r"""SharedSets are used for sharing criterion exclusions across
    multiple campaigns.


    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        resource_name (str):
            Immutable. The resource name of the shared set. Shared set
            resource names have the form:

            ``customers/{customer_id}/sharedSets/{shared_set_id}``
        id (int):
            Output only. The ID of this shared set. Read
            only.

            This field is a member of `oneof`_ ``_id``.
        type_ (google.ads.googleads.v19.enums.types.SharedSetTypeEnum.SharedSetType):
            Immutable. The type of this shared set: each
            shared set holds only a single kind of resource.
            Required. Immutable.
        name (str):
            The name of this shared set. Required.
            Shared Sets must have names that are unique
            among active shared sets of the same type.
            The length of this string should be between 1
            and 255 UTF-8 bytes, inclusive.

            This field is a member of `oneof`_ ``_name``.
        status (google.ads.googleads.v19.enums.types.SharedSetStatusEnum.SharedSetStatus):
            Output only. The status of this shared set.
            Read only.
        member_count (int):
            Output only. The number of shared criteria
            within this shared set. Read only.

            This field is a member of `oneof`_ ``_member_count``.
        reference_count (int):
            Output only. The number of campaigns
            associated with this shared set. Read only.

            This field is a member of `oneof`_ ``_reference_count``.
    """

    resource_name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    id: int = proto.Field(
        proto.INT64,
        number=8,
        optional=True,
    )
    type_: shared_set_type.SharedSetTypeEnum.SharedSetType = proto.Field(
        proto.ENUM,
        number=3,
        enum=shared_set_type.SharedSetTypeEnum.SharedSetType,
    )
    name: str = proto.Field(
        proto.STRING,
        number=9,
        optional=True,
    )
    status: shared_set_status.SharedSetStatusEnum.SharedSetStatus = proto.Field(
        proto.ENUM,
        number=5,
        enum=shared_set_status.SharedSetStatusEnum.SharedSetStatus,
    )
    member_count: int = proto.Field(
        proto.INT64,
        number=10,
        optional=True,
    )
    reference_count: int = proto.Field(
        proto.INT64,
        number=11,
        optional=True,
    )


__all__ = tuple(sorted(__protobuf__.manifest))
